@@ -31,7 +31,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/common/packet.h,v 1.93 2003/02/02 22:33:53 xuanc Exp $ (LBL)
+ * @(#) $Header: /cvsroot/nsnam/ns-2/common/packet.h,v 1.100 2006/02/22 13:32:23 mahrenho Exp $ (LBL)
  */
 
 #ifndef ns_packet_h
@@ -57,9 +57,11 @@
 #define HDR_MAC_TDMA(p) ((hdr_mac_tdma *)hdr_mac::access(p))
 #define HDR_SMAC(p)     ((hdr_smac *)hdr_mac::access(p))
 #define HDR_LL(p)       (hdr_ll::access(p))
+#define HDR_HDLC(p)     ((hdr_hdlc *)hdr_ll::access(p))
 #define HDR_IP(p)       (hdr_ip::access(p))
 #define HDR_RTP(p)      (hdr_rtp::access(p))
 #define HDR_TCP(p)      (hdr_tcp::access(p))
+#define HDR_SCTP(p)     (hdr_sctp::access(p))
 #define HDR_SR(p)       (hdr_sr::access(p))
 #define HDR_TFRC(p)     (hdr_tfrc::access(p))
 #define HDR_TORA(p)     (hdr_tora::access(p))
@@ -153,6 +155,21 @@ enum packet_t {
 	PT_LMS,
 	PT_LMS_SETUP,
 
+	PT_SCTP,
+	PT_SCTP_APP1,
+
+	// SMAC packet
+	PT_SMAC,
+
+	// XCP packet
+	PT_XCP,
+	
+	// HDLC packet
+	PT_HDLC,
+
+	// Bell Labs Traffic Trace Type (PackMime OL)
+	PT_BLTRACE, 
+
 	// insert new packet types here
 	PT_NTYPE // This MUST be the LAST one
 };
@@ -237,6 +254,21 @@ public:
 		name_[PT_LMS]="LMS";
 		name_[PT_LMS_SETUP]="LMS_SETUP";
 
+		name_[PT_SCTP]= "sctp";
+ 		name_[PT_SCTP_APP1] = "sctp_app1";
+		
+		// smac
+		name_[PT_SMAC]="smac";
+
+		// HDLC
+		name_[PT_HDLC]="HDLC";
+
+		// XCP
+		name_[PT_XCP]="xcp";
+
+		// Bell Labs (PackMime OL)
+		name_[PT_BLTRACE]="BellLabsTrace";
+		
 		name_[PT_NTYPE]= "undefined";
 	}
 	const char* name(packet_t p) const { 
@@ -249,8 +281,11 @@ public:
 			 (type) == PT_CBR || \
 			 (type) == PT_AUDIO || \
 			 (type) == PT_VIDEO || \
-			 (type) == PT_ACK \
-			 );
+			 (type) == PT_ACK || \
+			 (type) == PT_SCTP || \
+			 (type) == PT_SCTP_APP1 || \
+			 (type) == PT_HDLC \
+			);
 	}
 private:
 	static char* name_[PT_NTYPE+1];
@@ -264,11 +299,14 @@ extern p_info packet_info; /* map PT_* to string name */
                             (type) == PT_CBR || \
                             (type) == PT_AUDIO || \
                             (type) == PT_VIDEO || \
-                            (type) == PT_ACK \
+                            (type) == PT_ACK || \
+                            (type) == PT_SCTP || \
+                            (type) == PT_SCTP_APP1 \
                             )
 
 
-#define OFFSET(type, field)	((int) &((type *)0)->field)
+//#define OFFSET(type, field)	((long) &((type *)0)->field)
+#define OFFSET(type, field) ( (char *)&( ((type *)256)->field )  - (char *)256)
 
 class PacketData : public AppData {
 public:
