@@ -31,12 +31,12 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/common/scheduler.cc,v 1.70 2002/08/02 01:35:20 yuri Exp $
+ * @(#) $Header: /cvsroot/nsnam/ns-2/common/scheduler.cc,v 1.74 2006/02/21 15:20:18 mahrenho Exp $
  */
 
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/common/scheduler.cc,v 1.70 2002/08/02 01:35:20 yuri Exp $ (LBL)";
+    "@(#) $Header: /cvsroot/nsnam/ns-2/common/scheduler.cc,v 1.74 2006/02/21 15:20:18 mahrenho Exp $ (LBL)";
 #endif
 
 #include <stdlib.h>
@@ -89,6 +89,7 @@ Scheduler::schedule(Handler* h, Event* e, double delay)
 			"  Don't DO that.\n");
 		abort();
 	};
+	
 	if (e->uid_ > 0) {
 		printf("Scheduler: Event UID not valid!\n\n");
 		abort();
@@ -285,7 +286,7 @@ Scheduler::dumpq()
 	while ((p = deque()) != NULL) {
 		printf("t:%f uid: ", p->time_);
 		printf(UID_PRINTF_FORMAT, p->uid_);
-		printf(" handler: %p\n", p->handler_);
+		printf(" handler: %p\n", reinterpret_cast<void *>(p->handler_) );
 	}
 }
 
@@ -459,7 +460,7 @@ Heap::heap_insert(heap_key_t key, void* elem)
 	h_elems[i].he_s_key= h_s_key++;
 	h_elems[i].he_elem = elem;
 	return;
-};
+}
 		
 /*
  * void *heap_extract_min(Heap *h)
@@ -703,7 +704,7 @@ CalendarScheduler::head()
 	if (qsize_ == 0)
 		return NULL;
 
-	int l, i = lastbucket_;
+	int l = -1, i = lastbucket_;
 	int lastbucket_dec = (lastbucket_) ? lastbucket_ - 1 : nbuckets_ - 1;
 	double diff;
 	Event *e, *min_e = NULL;
@@ -753,6 +754,7 @@ do { 								\
 	 * still want to advance lastbucket_ and cal_clock_ to save
 	 * time when deque() follows. 
 	 */
+	assert (l != -1);
 	lastbucket_ = l;
  	cal_clock_  = e->time_;
 	
@@ -862,7 +864,7 @@ double
 CalendarScheduler::newwidth(int newsize)
 {
 	int i;
-	short max_bucket = 0; // index of the fullest bucket
+	int max_bucket = 0; // index of the fullest bucket
 	for (i = 1; i < nbuckets_; ++i) {
 		if (buckets_[i].count_ > buckets_[max_bucket].count_)
 			max_bucket = i;
