@@ -1,21 +1,50 @@
+
 /*
- * Copyright (c) 2001 University of Southern California.
- * All rights reserved.                                            
- *                                                                
- * Redistribution and use in source and binary forms are permitted
- * provided that the above copyright notice and this paragraph are
- * duplicated in all such forms and that any documentation, advertising
- * materials, and other materials related to such distribution and use
- * acknowledge that the software was developed by the University of
- * Southern California, Information Sciences Institute.  The name of the
- * University may not be used to endorse or promote products derived from
- * this software without specific prior written permission.
- * 
- * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
- * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * pgm-agent.cc
+ * Copyright (C) 2001 by the University of Southern California
+ * $Id: pgm-agent.cc,v 1.10 2006/02/21 15:20:19 mahrenho Exp $
  *
- **
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * The copyright of this module includes the following
+ * linking-with-specific-other-licenses addition:
+ *
+ * In addition, as a special exception, the copyright holders of
+ * this module give you permission to combine (via static or
+ * dynamic linking) this module with free software programs or
+ * libraries that are released under the GNU LGPL and with code
+ * included in the standard release of ns-2 under the Apache 2.0
+ * license or under otherwise-compatible licenses with advertising
+ * requirements (or modified versions of such code, with unchanged
+ * license).  You may copy and distribute such a system following the
+ * terms of the GNU GPL for this module and the licenses of the
+ * other code concerned, provided that you include the source code of
+ * that other code when and as the GNU GPL requires distribution of
+ * source code.
+ *
+ * Note that people who make modified versions of this module
+ * are not obligated to grant this special exception for their
+ * modified versions; it is their choice whether to do so.  The GNU
+ * General Public License gives permission to release a modified
+ * version without this exception; this exception also makes it
+ * possible to release a modified version which carries forward this
+ * exception.
+ *
+ */
+
+/*
  * Pragmatic General Multicast (PGM), Reliable Multicast
  *
  * pgm-agent.cc
@@ -275,7 +304,7 @@ protected:
 
   void print_stats();
 
-  void PgmAgent::trace_event(char *evType, double evTime);
+  void trace_event(char *evType, double evTime);
 
 #ifdef PGM_DEBUG
   void display_packet(Packet *pkt);
@@ -526,7 +555,7 @@ void PgmAgent::handle_odata(Packet *pkt)
   // Pass the ODATA along to the rest of the multicast group.  ODATA
   // does not cancel NAK forwarding.
 
-  hdr_cmn *hc = HDR_CMN(pkt);
+  //hdr_cmn *hc = HDR_CMN(pkt);
 
   send(pkt, 0);
 }
@@ -536,7 +565,7 @@ void PgmAgent::handle_rdata(Packet *pkt)
 
   // Look for the TSI for this RDATA packet.
   hdr_pgm *hp = HDR_PGM(pkt);
-  hdr_ip *hip = HDR_IP(pkt);
+  //  hdr_ip *hip = HDR_IP(pkt);
 
   StateInfo *state = find_TSI(hp->tsi_);
   if (state == NULL) {
@@ -577,7 +606,7 @@ void PgmAgent::handle_rdata(Packet *pkt)
 
   trace_event("SEND RDATA", 0); //Repair is being forwarded
 
-  hdr_cmn *hc = HDR_CMN(pkt);
+  //  hdr_cmn *hc = HDR_CMN(pkt);
 
   if (!rstate->iface_list().empty()) {
     list<int>::iterator iter = rstate->iface_list().begin();
@@ -641,7 +670,7 @@ void PgmAgent::handle_nak(Packet *pkt)
   hdr_pgm *hp = HDR_PGM(pkt);
   hdr_pgm_nak *hpn = HDR_PGM_NAK(pkt);
   hdr_cmn *hc = HDR_CMN(pkt);
-  hdr_ip *hip = HDR_IP(pkt);
+  //  hdr_ip *hip = HDR_IP(pkt);
 
   // Check to see if there is a state control block for the given TSI.
   StateInfo *state = find_TSI(hp->tsi_);
@@ -977,7 +1006,7 @@ NsObject* PgmAgent::iface2link (int iface)
 
         sprintf (wrk, "[%s set node_] ifaceGetOutLink %d", name (), iface);
         tcl.evalc (wrk);
-        char* result = tcl.result ();
+        const char* result = tcl.result ();
 #ifdef PGM_DEBUG
 printf ("[iface2link] agent %s\n", result);
 #endif
@@ -989,11 +1018,11 @@ NsObject* PgmAgent::pkt2agent (Packet *pkt)
 {
         Tcl&            tcl = Tcl::instance();
         char            wrk[64];
-        char            *result;
+        const char     *result;
         int             port;
         NsObject*       agent;
         hdr_ip*         ih = HDR_IP(pkt);
-        nsaddr_t        src = ih->saddr();
+        //nsaddr_t        src = ih->saddr();
 
         port = ih->sport();
 
@@ -1016,7 +1045,7 @@ StateInfo * PgmAgent::find_TSI(ns_addr_t &tsi)
   // Use the TSI from the SPM packet and locate the proper state block.
   list<StateInfo>::iterator iter = state_list_.begin();
   while(iter != state_list_.end()) {
-    if ( (*iter).tsi() == tsi ) {
+    if ( (*iter).tsi().isEqual (tsi) ) {
       return &(*iter);
     }
     iter++;
