@@ -57,7 +57,7 @@
 
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/queue/red.cc,v 1.76 2003/01/28 19:50:51 sfloyd Exp $ (LBL)";
+     "@(#) $Header: /cvsroot/nsnam/ns-2/queue/red.cc,v 1.83 2006/12/17 15:21:59 mweigle Exp $ (LBL)";
 #endif
 
 #include <math.h>
@@ -93,8 +93,10 @@ REDQueue::REDQueue() {
 /*
  * modified to enable instantiation with special Trace objects - ratul
  */
-REDQueue::REDQueue(const char * trace) : link_(NULL), de_drop_(NULL), EDTrace(NULL), tchan_(0), idle_(1)
+REDQueue::REDQueue(const char * trace) : link_(NULL), de_drop_(NULL), EDTrace(NULL), tchan_(0), idle_(1), idletime_(0.0)
 {
+	initParams();
+	
 	//	printf("Making trace type %s\n", trace);
 	if (strlen(trace) >=20) {
 		printf("trace type too long - allocate more space to traceType in red.h and recompile\n");
@@ -106,7 +108,9 @@ REDQueue::REDQueue(const char * trace) : link_(NULL), de_drop_(NULL), EDTrace(NU
 	//	_RENAMED("queue-in-bytes_", "queue_in_bytes_");
 
 	bind("thresh_", &edp_.th_min_pkts);		    // minthresh
+	bind("thresh_queue_", &edp_.th_min);
 	bind("maxthresh_", &edp_.th_max_pkts);	    // maxthresh
+	bind("minthresh_queue_", &edp_.th_max);
 	bind("mean_pktsize_", &edp_.mean_pktsize);  // avg pkt size
 	bind("idle_pktsize_", &edp_.idle_pktsize);  // avg pkt size for idles
 	bind("q_weight_", &edp_.q_w);		    // for EWMA
@@ -122,6 +126,7 @@ REDQueue::REDQueue(const char * trace) : link_(NULL), de_drop_(NULL), EDTrace(NU
 	bind_bool("wait_", &edp_.wait);
 	bind("linterm_", &edp_.max_p_inv);
 	bind("mark_p_", &edp_.mark_p);
+	bind_bool("use_mark_p_", &edp_.use_mark_p);
 	bind_bool("setbit_", &edp_.setbit);	    // mark instead of drop
 	bind_bool("gentle_", &edp_.gentle);         // increase the packet
 						    // drop prob. slowly
@@ -205,8 +210,61 @@ void REDQueue::initialize_params()
 		edp_.th_max_pkts = 3.0 * edp_.th_min_pkts;
         //printf("th_min_pkts: %7.5f th_max_pkts: %7.5f\n", edp_.th_min_pkts, edp_.th_max);
 	//printf("q_w: %7.5f\n", edp_.q_w);
+	if (edp_.bottom == 0) {
+		edp_.bottom = 0.01;
+		// Set bottom to at most 1/W, for W the delay-bandwidth 
+		//   product in packets for a connection with this bandwidth,
+		//   1000-byte packets, and 100 ms RTTs.
+		// So W = 0.1 * link_->bandwidth() / 8000 
+		double bottom1 = 80000.0/link_->bandwidth();
+		if (bottom1 < edp_.bottom) 
+			edp_.bottom = bottom1;
+		//printf("bottom: %9.7f\n", edp_.bottom);
+	}
 }
 
+void REDQueue::initParams() 
+{
+	edp_.mean_pktsize = 0;
+	edp_.idle_pktsize = 0;
+	edp_.bytes = 0;
+	edp_.wait = 0;
+	edp_.setbit = 0;
+	edp_.gentle = 0;
+	edp_.th_min = 0.0;
+	edp_.th_min_pkts = 0.0;
+	edp_.th_max = 0.0;
+	edp_.th_max_pkts = 0.0;
+	edp_.max_p_inv = 0.0;
+	edp_.q_w = 0.0;
+	edp_.adaptive = 0;
+	edp_.cautious = 0;
+	edp_.alpha = 0.0;
+	edp_.beta = 0.0;
+	edp_.interval = 0.0;
+	edp_.targetdelay = 0.0;
+	edp_.top = 0.0;
+	edp_.bottom = 0.0;
+	edp_.feng_adaptive = 0;
+	edp_.ptc = 0.0;
+	edp_.delay = 0.0;
+	
+	edv_.v_ave = 0.0;
+	edv_.v_prob1 = 0.0;
+	edv_.v_slope = 0.0;
+	edv_.v_prob = 0.0;
+	edv_.v_a = 0.0;
+	edv_.v_b = 0.0;
+	edv_.v_c = 0.0;
+	edv_.v_d = 0.0;
+	edv_.count = 0;
+	edv_.count_bytes = 0;
+	edv_.old = 0;
+	edv_.cur_max_p = 1.0;
+	edv_.lastset = 0;
+}
+
+
 void REDQueue::reset()
 {
 	
@@ -219,7 +277,7 @@ void REDQueue::reset()
 	 * accordingly.
 	 */
         if (link_) {
-		edp_.ptc = link_->bandwidth() / (8. * edp_.mean_pktsize);
+		edp_.ptc = link_->bandwidth() / (8.0 * edp_.mean_pktsize);
 		initialize_params();
 	}
 	if (edp_.th_max_pkts == 0) 
@@ -243,13 +301,22 @@ void REDQueue::reset()
 	edv_.count = 0;
 	edv_.count_bytes = 0;
 	edv_.old = 0;
-	edv_.v_a = 1 / (edp_.th_max - edp_.th_min);
+	double th_diff = (edp_.th_max - edp_.th_min);
+	if (th_diff == 0) { 
+		//XXX this last check was added by a person who knows
+		//nothing of this code just to stop FP div by zero.
+		//Values for thresholds were equal at time 0.  If you
+		//know what should be here, please cleanup and remove
+		//this comment.
+		th_diff = 1.0; 
+	}
+	edv_.v_a = 1.0 / th_diff;
 	edv_.cur_max_p = 1.0 / edp_.max_p_inv;
-	edv_.v_b = - edp_.th_min / (edp_.th_max - edp_.th_min);
+	edv_.v_b = - edp_.th_min / th_diff;
 	edv_.lastset = 0.0;
 	if (edp_.gentle) {
 		edv_.v_c = ( 1.0 - edv_.cur_max_p ) / edp_.th_max;
-		edv_.v_d = 2 * edv_.cur_max_p - 1.0;
+		edv_.v_d = 2.0 * edv_.cur_max_p - 1.0;
 	}
 
 	idle_ = 1;
@@ -413,7 +480,7 @@ REDQueue::modify_p(double p, int count, int count_bytes, int bytes,
 		if (count1 * p < 1.0)
 			p = 0.0;
 		else if (count1 * p < 2.0)
-			p /= (2 - count1 * p);
+			p /= (2.0 - count1 * p);
 		else
 			p = 1.0;
 	} else {
@@ -423,7 +490,9 @@ REDQueue::modify_p(double p, int count, int count_bytes, int bytes,
 			p = 1.0;
 	}
 	if (bytes && p < 1.0) {
-		p = p * size / mean_pktsize;
+		p = (p * size) / mean_pktsize;
+		//p = p * (size / mean_pktsize);
+
 	}
 	if (p > 1.0)
 		p = 1.0;
@@ -558,6 +627,7 @@ void REDQueue::enque(Packet* pkt)
 	 * of time we've been idle for
 	 */
 
+	/*  print_edp(); */
 	int m = 0;
 	if (idle_) {
 		// A packet that arrives to an idle queue will never
@@ -613,7 +683,8 @@ void REDQueue::enque(Packet* pkt)
 	curq_ = qlen;	// helps to trace queue during arrival, if enabled
 
 	if (qavg >= edp_.th_min && qlen > 1) {
-		if ((!edp_.gentle && qavg >= edp_.th_max) ||
+		if (!edp_.use_mark_p && 
+			(!edp_.gentle && qavg >= edp_.th_max) ||
 			(edp_.gentle && qavg >= 2 * edp_.th_max)) {
 			droptype = DTYPE_FORCED;
 		} else if (edv_.old == 0) {
@@ -747,7 +818,7 @@ int REDQueue::command(int argc, const char*const* argv)
 			// set ptc now
 			link_ = del;
 			edp_.ptc = link_->bandwidth() /
-				(8. * edp_.mean_pktsize);
+				(8.0 * edp_.mean_pktsize);
 			edp_.delay = link_->delay();
 			if (
 			  (edp_.q_w <= 0.0 || edp_.th_min_pkts == 0 ||
@@ -802,7 +873,8 @@ int REDQueue::command(int argc, const char*const* argv)
 void
 REDQueue::trace(TracedVar* v)
 {
-	char wrk[500], *p;
+	char wrk[500];
+	const char *p;
 
 	if (((p = strstr(v->name(), "ave")) == NULL) &&
 	    ((p = strstr(v->name(), "prob")) == NULL) &&
@@ -841,6 +913,7 @@ void REDQueue::print_edp()
 	printf("max_p: %f, qw: %f, ptc: %f\n",
 		(double) edv_.cur_max_p, edp_.q_w, edp_.ptc);
 	printf("qlim: %d, idletime: %f\n", qlim_, idletime_);
+	printf("mark_p: %f, use_mark_p: %d\n", edp_.mark_p, edp_.use_mark_p);
 	printf("=========\n");
 }
 
