@@ -21,6 +21,9 @@
 #    used to endorse or promote products derived from this software without
 #    specific prior written permission.
 # 
+
+# @(#) $Header: /cvsroot/nsnam/ns-2/tcl/lib/ns-default.tcl,v 1.369 2006/10/23 05:33:16 sallyfloyd Exp $
+
 # THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 # ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -33,9 +36,6 @@
 # OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 # SUCH DAMAGE.
 #
-# @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/tcl/lib/ns-default.tcl,v 1.310 2003/02/13 03:11:00 buchheim Exp $
-
-
 #
 #
 # Set up all the default paramters.  Each default parameter
@@ -47,10 +47,10 @@
 ########################################################
 
 # Debojyoti added this
-
 Simulator set useasim_ 1
 Asim set debug_ false
 
+set MAXSEQ 1073741824
 # Increased Floating Point Precision
 set tcl_precision 17
 
@@ -61,9 +61,12 @@ Trace set src_ -1
 Trace set dst_ -1
 Trace set callback_ 0
 Trace set show_tcphdr_ 0
+Trace set show_sctphdr_ 0
 Trace set debug_ false
 
+
 CMUTrace set debug_ false
+CMUTrace set show_sctphdr_ 0
 CMUTrace set radius_scaling_factor_ 1.0
 CMUTrace set duration_scaling_factor_ 3.0e4
 
@@ -109,14 +112,22 @@ CMUPriQueue set debug_ false
 Queue/dsRED set numQueues_ 4
 Queue/dsRED set ecn_ 0
 
-Queue/RED set bytes_ false
-Queue/RED set queue_in_bytes_ false
+# XXX Temporary fix XXX
+# support only xcp flows; set to 1 when supporting both tcp and xcp flows; temporary fix for allocating link BW between xcp and tcp queues until dynamic queue weights come into effect. This fix should then go away
+Queue/XCP set tcp_xcp_on_ 0  ;
+
+Queue/RED set bytes_ true ;		# default changed on 10/11/2004.
+Queue/RED set queue_in_bytes_ true ;	# default changed on 10/11/2004.
 # Queue/RED set thresh_ 5
 Queue/RED set thresh_ 0
 # Queue/RED/thresh_ was changed on 12/29/01, for automatic configuration.
 # Queue/RED set maxthresh_ 15
 Queue/RED set maxthresh_ 0
 # Queue/RED/maxthresh_ was changed on 12/29/01, for automatic configuration.
+Queue/RED set thresh_queue_ [Queue set limit_]
+# Note from Sally: I don't see that thresh_queue_ is used for anything.
+Queue/RED set minthresh_queue_ [Queue set limit_]
+# Note from Sally: I don't see that minthresh_queue_ is used for anything.
 Queue/RED set mean_pktsize_ 500
 Queue/RED set idle_pktsize_ 100
 # Queue/RED set q_weight_ 0.002
@@ -125,6 +136,9 @@ Queue/RED set q_weight_ -1
 Queue/RED set wait_ true
 Queue/RED set linterm_ 10
 Queue/RED set mark_p_ 0.1
+Queue/RED set use_mark_p_ true
+# Queue/RED/use_mark_p_ was added on 11/23/05.
+# Queue/RED/use_mark_p_ was changed to true on 12/22/05.
 Queue/RED set setbit_ false
 Queue/RED set gentle_ true
 ### Queue/RED/gentle_ was changed from false to true on Apr 23, 2001.
@@ -146,7 +160,9 @@ Queue/RED set adaptive_ 0
 Queue/RED set interval_ 0.5
 Queue/RED set targetdelay_ 0.005
 Queue/RED set top_ 0.5
-Queue/RED set bottom_ 0.01
+Queue/RED set bottom_ 0
+### Queue/RED/bottom_ was changed from 0.01 to 0 on June 23, 2004,
+###   for automatic configuration.
 Queue/RED set cautious_ 0
 Queue/RED set feng_adaptive_ 0
 
@@ -279,6 +295,13 @@ QueueMonitor set pdrops_ 0
 QueueMonitor set pmarks_ 0
 QueueMonitor set bdrops_ 0
 
+QueueMonitor set qs_pkts_ 0
+QueueMonitor set qs_bytes_ 0
+QueueMonitor set qs_drops_ 0
+
+QueueMonitor set first_pkt_ 0
+QueueMonitor set last_pkt_ 0
+
 #added for keeping RTT stats
 QueueMonitor set keepRTTstats_ 0
 QueueMonitor set maxRTT_ 5
@@ -333,7 +356,9 @@ QueueMonitor/ED/Flow/RedPD set auto_ 0
 DelayLink set bandwidth_ 1.5Mb
 DelayLink set delay_ 100ms
 DelayLink set debug_ false
-
+DelayLink set avoidReordering_ false ;	# Added 3/27/2003.
+					# Set to true to avoid reordering when
+					#   changing link bandwidth or delay.
 DynamicLink set status_ 1
 DynamicLink set debug_ false
 
@@ -386,6 +411,9 @@ ErrorModel/Trace set loss_ 0
 ErrorModel/Periodic set period_ 1.0
 ErrorModel/Periodic set offset_ 0.0
 ErrorModel/Periodic set burstlen_ 0.0
+ErrorModel/Periodic set default_drop_ 0 ;	# set to 1 for default-drop,
+						# to drop all but last pkt
+						# in period_
 ErrorModel/MultiState set curperiod_ 0.0
 ErrorModel/MultiState set sttype_ pkt
 ErrorModel/MultiState set texpired_ 0
@@ -463,6 +491,17 @@ Application/Traffic/CBR set maxpkts_ 268435456; # 0x10000000
 
 Application/Telnet set interval_ 1.0
 
+# Default config based on data for slammer worm
+Application/Worm set ScanRate 4000
+Application/Worm set ScanPort 1434
+Application/Worm set ScanPacketSize 404
+Application/Worm/An set TimeStep 1
+
+Application/SctpApp1 set interval_ 1.0
+Application/SctpApp1 set numStreams_ 1
+Application/SctpApp1 set numUnreliable_ 0
+Application/SctpApp1 set reliability_ 0
+
 RandomVariable/Uniform set min_ 0.0
 RandomVariable/Uniform set max_ 1.0
 RandomVariable/Exponential set avg_ 1.0
@@ -481,6 +520,8 @@ RandomVariable/Normal set avg_ 0.0
 RandomVariable/Normal set std_ 1.0
 RandomVariable/LogNormal set avg_ 1.0
 RandomVariable/LogNormal set std_ 1.0
+RandomVariable/Weibull set scale_ 1.0
+RandomVariable/Weibull set shape_ 1.0
 
 ADC/MS set debug_ false
 ADC/HB set debug_ false
@@ -543,6 +584,11 @@ Simulator set mobile_ip_ 0			 ;# flag for mobileIP
 Simulator set routingAgent_ ""
 Simulator set addressType_   ""
 Simulator set MovementTrace_ OFF
+
+# change wrt Mike's code
+Simulator set EotTrace_ OFF
+
+
 # This flag should be initially empty. It will be set to either ON or OFF
 # by Simulator::create-wireless-node{}. 
 Simulator set IMEPFlag_ ""
@@ -604,6 +650,7 @@ GAFPartner set debug_ false
 
 Mac set debug_ false
 ARPTable set debug_ false
+ARPTable set avoidReordering_ false ; #not used
 God set debug_ false
 
 Mac/Tdma set slot_packet_len_	1500
@@ -613,9 +660,27 @@ LL set mindelay_                50us
 LL set delay_                   25us
 LL set bandwidth_               0       ;# not used
 LL set debug_ false
+LL set avoidReordering_ false ;	#not used 
 
 Snoop set debug_ false
 
+#change wrt Mike's code
+ # 802.11 MIB parameters
+ #
+
+ Mac/802_11 set CWMin_         31
+ Mac/802_11 set CWMax_         1023
+ Mac/802_11 set SlotTime_      0.000020        ;# 20us
+ Mac/802_11 set SIFS_          0.000010        ;# 10us
+ Mac/802_11 set PreambleLength_        144             ;# 144 bit
+ Mac/802_11 set PLCPHeaderLength_      48              ;# 48 bits
+ Mac/802_11 set PLCPDataRate_  1.0e6           ;# 1Mbps
+ Mac/802_11 set RTSThreshold_  0               ;# bytes
+ Mac/802_11 set ShortRetryLimit_       7               ;# retransmittions
+ Mac/802_11 set LongRetryLimit_        4               ;# retransmissions
+
+Mac/802_11 set bugFix_timer_ true;         # fix for when RTS/CTS not used
+# details at http://www.dei.unipd.it/wdyn/?IDsezione=2435
 
 # 
 # Support for Abstract LAN
@@ -629,6 +694,14 @@ Mac set abstract_ false
 #
 Mac/802_3 set trace_ false
 
+# Turning on/off sleep-wakeup cycles for SMAC
+Mac/SMAC set syncFlag_ 1
+                                                                                                                
+# Nodes synchronize their schedules in SMAC
+Mac/SMAC set selfConfigFlag_ 1
+
+# Default duty cycle in SMAC
+Mac/SMAC set dutyCycle_ 10                                                                                                                
 #
 # Unity gain, omni-directional antennas
 # Set up the antennas to be centered in the node and 1.5 meters above it
@@ -638,6 +711,8 @@ Antenna/OmniAntenna set Z_ 1.5
 Antenna/OmniAntenna set Gt_ 1.0
 Antenna/OmniAntenna set Gr_ 1.0
 
+Phy set debug_ false
+
 # Initialize the SharedMedia interface with parameters to make
 # it work like the 914MHz Lucent WaveLAN DSSS radio interface
 Phy/WirelessPhy set CPThresh_ 10.0
@@ -647,16 +722,8 @@ Phy/WirelessPhy set bandwidth_ 2e6
 Phy/WirelessPhy set Pt_ 0.28183815
 Phy/WirelessPhy set freq_ 914e+6
 Phy/WirelessPhy set L_ 1.0  
-Phy/WirelessPhy set debug_ false
 
 Phy/WiredPhy set bandwidth_ 10e6
-Phy/WiredPhy set debug_ false
-Phy/Repeater set debug_ false
-LanRouter set debug_ false
-
-Phy/Sat set debug_ false
-Mac/Sat set debug_ false
-LL/Sat set debug_ false
 
 # Shadowing propagation model
 Propagation/Shadowing set pathlossExp_ 2.0
@@ -694,19 +761,84 @@ Agent/UDP set packetSize_ 1000
 Agent/UDP instproc done {} { }
 Agent/UDP instproc process_data {from data} { }
 
+Agent/SCTP set debugMask_ 0             ;# all debugging off by default
+Agent/SCTP set debugFileIndex_ -1       ;# default outputs to stderr
+Agent/SCTP set associationMaxRetrans_ 10;# 10 attempts
+Agent/SCTP set pathMaxRetrans_ 5        ;# 5 attempts (per destination)
+Agent/SCTP set changePrimaryThresh_ -1  ;# infinite (ie, never change primary
+Agent/SCTP set maxInitRetransmits_ 8    ;# 8 attempts
+Agent/SCTP set oneHeartbeatTimer_ 1     ;# single heartbeat timer for all dests
+Agent/SCTP set heartbeatInterval_ 30    ;# 30 secs
+Agent/SCTP set mtu_ 1500                ;# MTU of ethernet (most common)
+Agent/SCTP set initialRwnd_ 65536       ;# default inital receiver window
+Agent/SCTP set initialSsthresh_ 65536   ;# default inital ssthresh value
+Agent/SCTP set initialCwnd_ 2           ;# default cwnd = 2 * MTU
+Agent/SCTP set initialRto_ 3.0          ;# default initial RTO = 3 secs       
+Agent/SCTP set minRto_ 1.0              ;# default min RTO = 1 sec            
+Agent/SCTP set maxRto_ 60.0             ;# default max RTO = 60 secs          
+Agent/SCTP set fastRtxTrigger_ 4        ;# 4 missing reports trigger fast rtx
+Agent/SCTP set numOutStreams_ 1         ;# single stream default
+Agent/SCTP set numUnrelStreams_ 0       ;# by default all streams are reliable
+Agent/SCTP set reliability_ 0           ;# by default unrel streams have 0 rtx's
+Agent/SCTP set unordered_ 0             ;# by default all chunks are ordered
+Agent/SCTP set ipHeaderSize_ 20         ;# default is IPv4
+Agent/SCTP set dataChunkSize_ 1468      ;# restricted to 4 byte boundaries
+Agent/SCTP set useDelayedSacks_ 1       ;# rfc2960 says SHOULD use delayed sacks
+Agent/SCTP set sackDelay_ 0.200         ;# rfc2960 recommends 200 ms
+Agent/SCTP set useMaxBurst_ 1           ;# sctp implementors guide adds this var
+Agent/SCTP set rtxToAlt_ 1              ;# by default rtxs go to alternate dest
+Agent/SCTP set dormantAction_ 0		;# 0 = change dest, 1 = use primary, 2 = use last dest before dormant;
+                                                                             
+## These variables are for simulating reactive routing overheads (for         
+## MANETs, etc). This feature is turned off is delay is 0. The cache lifetime 
+## by default is just slightly larger than the default min RTO to avoid a "cache                                                                             
+## miss" after a single timeout event.
+Agent/SCTP set routeCalcDelay_ 0        ;# time to calculate a route          
+Agent/SCTP set routeCacheLifetime_ 1.2  ;# how long a route remains cached  
+
+Agent/SCTP set trace_all_ 0             ;# trace all vars ?
+
+## These variables are set because they have to be bound to be traceable.
+## This default does not matter to us at all.
+Agent/SCTP set cwnd_ 0                 ; 
+Agent/SCTP set rwnd_ 0                 ; 
+Agent/SCTP set rto_ 0                  ;
+Agent/SCTP set errorCount_ 0           ;
+Agent/SCTP set frCount_ 0              ;                                      
+Agent/SCTP set timeoutCount_ 0         ;                                      
+Agent/SCTP set rcdCount_ 0             ;# total count of route calc delays    
+
+Agent/SCTP/MultipleFastRtx set mfrCount_ 0                                    
+Agent/SCTP/MfrTimestamp set mfrCount_ 0    
+
+## CMT variables
+Agent/SCTP/CMT set useCmtReordering_ 1  ;# Turn ON CMT Reordering algo
+Agent/SCTP/CMT set useCmtCwnd_ 1        ;# Turn ON CMT cwnd growth algo
+Agent/SCTP/CMT set useCmtDelAck_ 1      ;# Turn ON CMT delayed ack algo
+Agent/SCTP/CMT set eCmtRtxPolicy_ 1     ;# Default policy = RTX_TO_SAME
+## CMT-PF variables
+Agent/SCTP/CMT set useCmtPF_ 0          ;# CMT-PF turned off
+Agent/SCTP/CMT set cmtPFCwnd_ 1	        ;# Cwnd in MTUs after HB-ACK (1 or 2)
+
 Agent/TCP set seqno_ 0
 Agent/TCP set t_seqno_ 0
 Agent/TCP set maxburst_ 0
+Agent/TCP set aggressive_maxburst_ 1 ;  # Added 2003/6/2.
+					# No change to default behavior.
 Agent/TCP set maxcwnd_ 0
 Agent/TCP set numdupacks_ 3
-Agent/TCP set numdupacksFrac_ 100 ;	# Added 2002/10/18.
-					# Set to 100000 for this to have
-					# no effect even for large windows.
+Agent/TCP set numdupacksFrac_ -1 ;	# Added 2002/10/18.
+					# Set to -1  for this to have
+					#   no effect even for large windows.
+					# Default changed to -1 on 2006/09/27.
+                                        # Set to 100 to have numdupacks_
+                                        #   up to 1/100-th of cwnd.
+Agent/TCP set exitFastRetrans_ true ;	# Added 2003/7/28.
+					# For clean exits of Fast Retransmit.
+					# False for old buggy behavior.
 Agent/TCP set window_ 20
-# Agent/TCP set windowInit_ 1
 Agent/TCP set windowInit_ 2 ;		# default changed on 2001/5/26.
 Agent/TCP set windowInitOption_ 1
-# Agent/TCP set syn_ false
 Agent/TCP set syn_ true ;		# default changed on 2001/5/17.
 Agent/TCP set windowOption_ 1
 Agent/TCP set windowConstant_ 4
@@ -718,32 +850,39 @@ Agent/TCP set l_parameter_ 1.0 ;  	# for binomial congestion control
 Agent/TCP set overhead_ 0
 Agent/TCP set ecn_ 0
 Agent/TCP set old_ecn_ 0
+Agent/TCP set bugfix_ss_ 1 ;		# Variable added on 2006/06/13
 Agent/TCP set packetSize_ 1000
 Agent/TCP set tcpip_base_hdr_size_ 40
 Agent/TCP set ts_option_size_ 10; 	# in bytes
 Agent/TCP set bugFix_ true
+Agent/TCP set bugFix_ack_ false ;       # Variable added on 2003/08/13
+					# To allow multiple Fast Retransmits
+Agent/TCP set bugFix_ts_ false ;	# Variable added on 2003/08/13
+					# To allow multiple Fast Retransmits
 Agent/TCP set lessCareful_ false ;	# for the Less Careful variant of
 					# bugFix_, just for illustration.
 Agent/TCP set timestamps_ false
+Agent/TCP set ts_resetRTO_ false ;	# Added 2003/07/24.
+					# Set to true to un-back-off RTO
+					#   after any valid RTT measurement.
 Agent/TCP set slow_start_restart_ true
 Agent/TCP set restart_bugfix_ true
-# Agent/TCP set tcpTick_ 0.1
 Agent/TCP set tcpTick_ 0.01 ;		# default changed on 2002/03/07
 					# to reflect a changing reality.
 Agent/TCP set maxrto_ 100000
-#Agent/TCP set minrto_ 0 ;		# Variable added on 2001/05/10
-Agent/TCP set minrto_ 1 ;		# Default set to 1 on 2001/05/14.
-					# Set to "0" to give old behavior.
+Agent/TCP set minrto_ 0.2 ;		# Default changed to 200ms on 
+					#  2004/10/14, to match values
+					#  used by many implementations.
 Agent/TCP set srtt_init_ 0
 Agent/TCP set rttvar_init_ 12
-Agent/TCP set rtxcur_init_ 6.0
+Agent/TCP set rtxcur_init_ 3.0 ;	# Default changed on 2006/01/21		
 Agent/TCP set T_SRTT_BITS 3
 Agent/TCP set T_RTTVAR_BITS 2
 Agent/TCP set rttvar_exp_ 2
+Agent/TCP set updated_rttvar_ true ;	# Variable added on 2006/1/21
 Agent/TCP set timerfix_ true ; 		# Variable added on 2001/05/11
  					# Set to "false" to give the old 
 					#  behavior. 
-# Agent/TCP set rfc2988_ false ;	# Variable added on 2002/03/07.
 Agent/TCP set rfc2988_ true ;		# Default set to "true" on 2002/03/07.
 					# Set rfc2988_ "true" to give RFC2988-
 					#  compliant behavior for timers.
@@ -752,7 +891,6 @@ Agent/TCP set noFastRetrans_ false
 Agent/TCP set partial_ack_ false ;	# Variable added on 2002/12/28.
 					# Set to "true" to ensure sending
 					#  a packet on a partial ACK.
-
 Agent/TCP set dupacks_ 0
 Agent/TCP set ack_ 0
 Agent/TCP set cwnd_ 0
@@ -763,26 +901,28 @@ Agent/TCP set srtt_ 0
 Agent/TCP set rttvar_ 0
 Agent/TCP set backoff_ 0
 Agent/TCP set maxseq_ 0
-# Agent/TCP set singledup_ 0
 Agent/TCP set singledup_ 1 ;		# default changed on 2001/11/28.
-Agent/TCP set precisionReduce_ false
+Agent/TCP set LimTransmitFix_ false ;	# added on 2003/03/31.
+Agent/TCP set precisionReduce_ true ;	# default changed on 2006/1/24.
 Agent/TCP set oldCode_ false
 Agent/TCP set useHeaders_ true ;	# default changed on 2001/11/28. 
 
-# These are all variables for experimental high-speed TCP.
-# Agent/TCP set low_window_ 31
-Agent/TCP set low_window_ 38
-# low_window_ was changed on 2002/8/12 from 31 to 38, for low_p_ of 0.001.
-# This should be an undetectable change in performance.
+# These are all used for high-speed TCP.
+Agent/TCP set low_window_ 38 ;		# default changed on 2002/8/12.		
 Agent/TCP set high_window_ 83000
 Agent/TCP set high_p_ 0.0000001
 Agent/TCP set high_decrease_ 0.1
 Agent/TCP set max_ssthresh_ 0
-Agent/TCP set cwnd_frac_ 0.0
-
+Agent/TCP set cwnd_range_ 0 ;		# cwnd_frac_ deleted on 6/6/04,
+					# replaced by cwnd_range_
 # For Quick-Start:
 Agent/TCP set rate_request_ 0
 Agent/TCP set qs_enabled_ false
+Agent/TCP set tcp_qs_recovery_ true ;	# Added on 2004/09/21, by Pasi.
+Agent/TCP set qs_request_mode_ 1
+Agent/TCP set qs_rtt_ 50
+Agent/TCP set print_request_ false
+Agent/TCP set qs_thresh_ 4
 
 # These are all variables for tracing information.
 Agent/TCP set ndatapack_ 0
@@ -793,6 +933,7 @@ Agent/TCP set nrexmitpack_ 0
 Agent/TCP set nrexmitbytes_ 0
 Agent/TCP set necnresponses_ 0
 Agent/TCP set ncwndcuts_ 0 
+Agent/TCP set ncwndcuts1_ 0
 
 Agent/TCP set trace_all_oneline_ false
 
@@ -800,6 +941,8 @@ Agent/TCP set QOption_ 0
 Agent/TCP set EnblRTTCtr_ 0
 Agent/TCP set control_increase_ 0
 
+Agent/TCP set SetCWRonRetransmit_ false ; # added on 2005/06/19.
+
 # XXX Generate nam trace or plain old text trace for variables. 
 # When it's true, generate nam trace.
 Agent/TCP set nam_tracevar_ false
@@ -807,6 +950,9 @@ Agent/TCP set nam_tracevar_ false
 Agent/TCP/Fack set ss-div4_ false
 Agent/TCP/Fack set rampdown_ false
 
+Agent/TCP/Reno/XCP set timestamps_ true
+Agent/TCP/FullTcp/Newreno/XCP set timestamps_ true
+
 Agent/TCP set eln_ 0
 Agent/TCP set eln_rxmit_thresh_ 1
 # Agent/TCP set delay_growth_ false
@@ -814,17 +960,23 @@ Agent/TCP set delay_growth_ true ;	# default changed on 2001/5/17.
 
 Agent/TCP set CoarseTimer_      0
 
+Agent/TCP set frto_enabled_	0 ;	# Added on 2004/10/26 for F-RTO
+Agent/TCP set sfrto_enabled_	0 ;	# Added on 2004/10/26 for F-RTO
+Agent/TCP set spurious_response_ 1 ;	# Added on 2004/10/26 for F-RTO
+
 Agent/TCPSink set sport_        0
 Agent/TCPSink set dport_        0         
 
 #XXX other kinds of sinks -> should reparent
 Agent/TCPSink set packetSize_ 40
 Agent/TCPSink set maxSackBlocks_ 3
-Agent/TCPSink set ts_echo_bugfix_ false
+Agent/TCPSink set ts_echo_bugfix_ true ;	# default changed, 2003/8/13
+Agent/TCPSink set ts_echo_rfc1323_ false ;	# default added, 2003/8/13
 Agent/TCPSink set generateDSacks_ false
 Agent/TCPSink set qs_enabled_ false
 Agent/TCPSink set RFC2581_immediate_ack_ true
 Agent/TCPSink set bytes_ 0
+Agent/TCPSink set ecn_syn_ false ;	# Added 2005/11/21 for SYN/ACK pkts.
 
 Agent/TCPSink/DelAck set interval_ 100ms
 catch {
@@ -833,14 +985,17 @@ catch {
 }
 Agent/TCPSink/Sack1/DelAck set interval_ 100ms
 
-# setting this to 1 implements some changes to reno 
-# proposed by Janey Hoe (other than fixing reno's
-# unnecessary retransmit timeouts)
+ # setting newreno_changes_ to 1 implements some changes to reno 
+ # proposed by Janey Hoe (other than fixing reno's
+ # unnecessary retransmit timeouts)
 Agent/TCP/Newreno set newreno_changes_ 0
-# setting this to 1 allows the retransmit timer to expire for
-# a window with many packet drops
-Agent/TCP/Newreno set newreno_changes1_ 0
-Agent/TCP/Newreno set partial_window_deflation_ 0
+ # setting newreno_changes1_ to 1 allows the retransmit timer to expire for
+ # a window with many packet drops
+ # Default changed to 1 on 5/5/03, to reflect RFC 2582.
+Agent/TCP/Newreno set newreno_changes1_ 1
+Agent/TCP/Newreno set partial_window_deflation_ 1 ; # Default changed to 1
+						    # on 5/5/03, to reflect
+						    # RFC 2582.
 Agent/TCP/Newreno set exit_recovery_fix_ 0
 
 Agent/TCP/Vegas set v_alpha_ 1
@@ -879,34 +1034,63 @@ Agent/TCP/RFC793edu set add793exponinc_    false
 
 Agent/TCP/FullTcp instproc done_data {} { }
 
-Agent/TFRC set packetSize_ 1000 
+# Dynamic state:
 Agent/TFRC set rate_ 0 
-Agent/TFRC set df_ 0.95 ;	# decay factor for accurate RTT estimate
-Agent/TFRC set tcp_tick_ 0.1 ;	
 Agent/TFRC set ndatapack_ 0 ;	# Number of packets sent
+Agent/TFRC set ndatabytes_ 0 ;	# Number of bytes sent
+Agent/TFRC set true_loss_rate_ 0.0 ; # For statistics only.
+# RTT:
 Agent/TFRC set srtt_init_ 0 ;	# Variables for tracking RTT	
 Agent/TFRC set rttvar_init_ 12  
 Agent/TFRC set rtxcur_init_ 6.0	
 Agent/TFRC set rttvar_exp_ 2	
 Agent/TFRC set T_SRTT_BITS 3	
 Agent/TFRC set T_RTTVAR_BITS 2	
+# VoIP mode:
+Agent/TFRC set voip_ 0 ;        # Added on 10/23/2004      
+				# 1 for voip mode.
+Agent/TFRC set voip_max_pkt_rate_ 100 ;  # Max rate in pps, for voip mode.
+Agent/TFRC set fsize_ 1460 ;	# Default size for large TCP packets. 
+				# Used for VoIP mode.
+Agent/TFRC set headersize_ 32 ; # Size for packet headers.
+# Variants in the TFRC algorithms:
+Agent/TFRC set rate_init_option_ 2 ;	# Added on 10/20/2004
+				# Set to 1 for backward compatibility. 
+				# Set to 2 for RFC 3390 initial rates
+				# Default changed on 10/21/2004.
+Agent/TFRC set slow_increase_ 1 ;	# Added on 10/20//2004
+				# Set to 1 for gradual rate changes.  
+				# This also gives backward compatibility.
+# Agent/TFRC set ss_changes_ 1 ;	# Deleted on 3/14//2006. 
+Agent/TFRC set maxHeavyRounds_ 1; # Number of rounds for sending rate allowed
+				  #  to be greater than twice receiving rate.
+Agent/TFRC set conservative_ 0 ;  # Set to true for a conservative 
+				  # response to heavy congestion.
+Agent/TFRC set scmult_ 1.5 ;	# self clocking parameter for conservative_
+Agent/TFRC set oldCode_ false ; # Set to 1 to use old code for datalimited
+				#   applications.
+				# Parameter added on 12/18/02.
+# Parameters:
+Agent/TFRC set packetSize_ 1000 
+Agent/TFRC set df_ 0.95 ;	# decay factor for accurate RTT estimate
+Agent/TFRC set tcp_tick_ 0.1 ;	
 Agent/TFRC set InitRate_ 300 ;	# Initial send rate	
 Agent/TFRC set overhead_ 0 ;	# If > 0, dither outgoing packets
 Agent/TFRC set ssmult_ 2 ; 	# Rate of increase during slow-start:
 Agent/TFRC set bval_ 1 ;	# Value of B for TCP formula
 Agent/TFRC set ca_ 1 ; 	 	# Enable Sqrt(RTT) congestion avoidance
 Agent/TFRC set printStatus_ 0 
-Agent/TFRC set maxHeavyRounds_ 1; # Number of rounds for sending rate allowed
-				  #  to be greater than twice receiving rate.
-Agent/TFRC set conservative_ 0 ;  # Set to true for a conservative 
-				  # response to heavy congestion.
-Agent/TFRC set scmult_ 1.5 ;	# self clocking parameter for conservative_
 Agent/TFRC set ecn_ 0 ;		# Set to 1 for ECN-capable connection.
+Agent/TFRC set minrto_ 0.0 ;	# Minimum RTO, for use in TCP equation.
+				# The default is not to use minrto_.
 Agent/TFRC set SndrType_ 0 ;    # Set to 1 to use data-producing applications
                                 #   such as FTP.
-Agent/TFRC set oldCode_ false ; # Set to 1 to use old code for datalimited
-				#   applications.
-				# Parameter added on 12/18/02.
+Agent/TFRC set maxqueue_ MAXSEQ ;  # queue from application.
+Agent/TFRC set rate_init_ 2 ;		# Added on 10/20/2004
+				# Set to 1 for backward compatibility. 
+				# Default changed on 10/21/2004.
+Agent/TFRC set useHeaders_ true ;	# Added on 2005/06/24. 
+Agent/TFRC set idleFix_ true ;	# Added on 2006/03/12.
 
 Agent/TFRCSink set packetSize_ 40
 Agent/TFRCSink set InitHistorySize_ 100000
@@ -916,6 +1100,9 @@ Agent/TFRCSink set NumSamples_ -1
 Agent/TFRCSink set discount_ 1;	# History Discounting
 Agent/TFRCSink set printLoss_ 0
 Agent/TFRCSink set smooth_ 1 ;	# smoother Average Loss Interval
+Agent/TFRCSink set ShortIntervals_ 0 ; #  For calculating loss event rates 
+                        	# for short loss intervals differently
+Agent/TFRCSink set ShortRtts_ 2 ; # Max num of RTTs in a short interval.
 Agent/TFRCSink set minlc_ 4
 Agent/TFRCSink set algo_ 1 ;  	# 1: algo from sigcomm paper 2: ewma 
 				# 3: fixed window
@@ -924,9 +1111,12 @@ Agent/TFRCSink set history_ 0.75 ;    # loss history for EWMA
 Agent/TFRCSink set PreciseLoss_ 1 ;   # 1 for more precise loss events
 				      # Introduced on 12/11/02, default 1.
 				      # No change in performance.
-Agent/TFRCSink set numPkts_ 3;	# Num non-sequential packets before loss
+Agent/TFRCSink set numPkts_ 1;	# Num non-sequential packets before loss
 				# Introduced on 12/12/02, with default 1.
 				# Default changed to 3 on 12/16/02.
+				# Default changed to 1 on 10/28/03 due
+				#    to a major bug.
+Agent/TFRCSink set bytes_ 0 ;	# For counting bytes received.
 
 if [TclObject is-class Agent/TCP/FullTcp] {
 	Agent/TCP/FullTcp set segsperack_ 1; # ACK frequency
@@ -947,6 +1137,8 @@ if [TclObject is-class Agent/TCP/FullTcp] {
 	Agent/TCP/FullTcp set open_cwnd_on_pack_ true; # ^ win on partial acks?
 	Agent/TCP/FullTcp set halfclose_ false; # do simplex closes (shutdown)?
 	Agent/TCP/FullTcp set nopredict_ false; # disable header prediction code?
+        Agent/TCP/FullTcp set ecn_syn_ false; # Make SYN/ACK packet ECN-Capable?
+        Agent/TCP/FullTcp set ecn_syn_wait_ false; # Wait after marked SYN/ACK? 
 
 	Agent/TCP/FullTcp/Newreno set recov_maxburst_ 2; # max burst dur recov
 
@@ -1023,6 +1215,8 @@ Agent/RTCP set seqno_ 0
 
 Agent/Message set packetSize_ 180
 
+Agent/MessagePassing set packetSize_ 1500
+
 Agent/LossMonitor set nlost_ 0
 Agent/LossMonitor set npkts_ 0
 Agent/LossMonitor set bytes_ 0
@@ -1148,29 +1342,21 @@ Agent/LMS/Receiver instproc done {} { }
 Agent/LMS/Receiver set packetSize_ $lsize
 
 # Following defaults defined for TCP Quick Start
-# http://www.icir.org/floyd/papers/draft-amit-quick-start-02.ps
-
-Agent/TCP/Newreno/QS set rbp_scale_ 0.75
-Agent/TCP/Newreno/QS set rbp_segs_actually_paced_ 0
-Agent/TCP/Newreno/QS set rbp_inter_pace_delay_ 0
-Agent/TCP/Newreno/QS set rate_request_ 128
-
+# http://www.icir.org/floyd/quickstart.html
 Agent/QSAgent set qs_enabled_ 1
 Agent/QSAgent set old_classifier_ 0
-Agent/QSAgent set state_delay_ 0.25
-Agent/QSAgent set alloc_rate_ 0.01
+Agent/QSAgent set state_delay_ 0.2 ;	# Changed from 0.25 to 0.2, 2/25/05.
+Agent/QSAgent set alloc_rate_ 0.85 ; 	# Changed from 0.6 to 0.85, 2/25/05.
+Agent/QSAgent set threshold_ 0.85 ; 	# Changed from 0.4 to 0.85, 2/25/05.
 Agent/QSAgent set max_rate_ 256
 Agent/QSAgent set mss_ [Agent/TCP set packetSize_]
-
-Agent/TCPSink/QS set sport_        0
-Agent/TCPSink/QS set dport_        0         
-
-Agent/TCPSink/QS set packetSize_ 40
-Agent/TCPSink/QS set maxSackBlocks_ 3
-Agent/TCPSink/QS set ts_echo_bugfix_ false
-Agent/TCPSink/QS set generateDSacks_ false
-Agent/TCPSink/QS set RFC2581_immediate_ack_ true
+Agent/QSAgent set rate_function_ 2
+Agent/QSAgent set algorithm_ 3	; 	# Changed from 2 to 3, 2/25/05.
 
 Queue set util_weight_ 0.8
+Queue set util_check_intv_ 0.2 ;	# Changed from 1 to 0.2, 2/25/05.
+Queue set util_records_ 5 ; 		# Changed from 0 to 5, 2/25/05.
 
 # Quick Start definitions end here
+
+Delayer set debug_ false
