@@ -32,7 +32,7 @@
 # SUCH DAMAGE.
 #
 
-# @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/tcl/lib/ns-lib.tcl,v 1.255 2003/01/16 21:06:02 buchheim Exp $
+# @(#) $Header: /cvsroot/nsnam/ns-2/tcl/lib/ns-lib.tcl,v 1.274 2007/01/30 05:00:51 tom_henderson Exp $
 
 
 #
@@ -157,6 +157,11 @@ source ns-diffusion.tcl
 source ../rtp/session-rtp.tcl
 source ../interface/ns-iface.tcl
 source ../lan/ns-mac.tcl
+
+# Added by Sushmita to support event tracing for mac-simple and 802.11
+source ../lan/ns-mac-simple.tcl
+source ../lan/ns-mac-802_11.tcl
+
 source ../lan/ns-ll.tcl
 source ../lan/vlan.tcl
 source ../lan/abslan.tcl
@@ -171,9 +176,10 @@ source ../ctr-mcast/CtrRPComp.tcl
 source ../mcast/BST.tcl
 source ../mcast/srm.tcl
 source ../mcast/srm-ssm.tcl
-source ../mcast/mftp_snd.tcl
-source ../mcast/mftp_rcv.tcl
-source ../mcast/mftp_rcv_stat.tcl
+# These files removed due to licensing conflicts
+# source ../mcast/mftp_snd.tcl
+# source ../mcast/mftp_rcv.tcl
+# source ../mcast/mftp_rcv_stat.tcl
 source ../mcast/McastMonitor.tcl
 source ../rlm/rlm.tcl
 source ../rlm/rlm-ns.tcl
@@ -218,6 +224,8 @@ if {[ns-hasSTL] == 1} {
 source ns-nix.tcl
 source ../pgm/ns-pgm.tcl
 source ../rtglib/ns-rtProtoLS.tcl
+source ../delaybox/delaybox.tcl
+source ../packmime/packmime.tcl
 }
 
 source ns-qsnode.tcl
@@ -317,11 +325,18 @@ Simulator instproc dumper obj {
 #                  -rxPower        (in W)
 #                  -txPower        (in W)
 #                  -idlePower      (in W)
+#
+#                  -sleepPower 	   (in W)
+#		   -sleepTime	   (in sec indicating when the node can start sleeping)
 #                  -agentTrace  ON
 #                  -routerTrace ON 
 #                  -macTrace OFF 
 #                  -toraDebug OFF                
 #                  -movementTrace OFF
+# change wrt Mike's code
+#                  -eotTrace OFF
+#                  -diffusionFilter "GradientFilter/OnePhasePullFilter/GeoRoutingFilter/RmstFilter/SourceRouteFilter/LogFilter/TagFilter"
+
 
 Simulator instproc addressType  {val} { $self set addressType_  $val }
 Simulator instproc adhocRouting  {val} { $self set routingAgent_  $val }
@@ -343,6 +358,12 @@ Simulator instproc initialEnergy  {val} { $self set initialEnergy_  $val }
 Simulator instproc txPower  {val} { $self set txPower_  $val }
 Simulator instproc rxPower  {val} { $self set rxPower_  $val }
 Simulator instproc idlePower  {val} { $self set idlePower_  $val }
+#
+Simulator instproc sleepPower  {val} { $self set sleepPower_  $val }
+Simulator instproc sleepTime  {val} { $self set sleepTime_  $val }
+Simulator instproc transitionPower  {val} { $self set transitionPower_  $val }
+Simulator instproc transitionTime  {val} { $self set transitionTime_  $val }
+#
 Simulator instproc IncomingErrProc  {val} { $self set inerrProc_  $val }
 Simulator instproc OutgoingErrProc  {val} { $self set outerrProc_  $val }
 Simulator instproc FECProc  {val} { $self set FECProc_  $val }
@@ -354,6 +375,11 @@ Simulator instproc toraDebug {val} {$self set toraDebug_ $val }
 Simulator instproc satNodeType {val} {$self set satNodeType_ $val}
 Simulator instproc downlinkBW {val} {$self set downlinkBW_ $val}
 Simulator instproc stopTime {val} {$self set stopTime_ $val}
+
+# change wrt Mike's code
+Simulator instproc eotTrace  {val} { $self set eotTrace_  $val }
+Simulator instproc diffusionFilter {val} {$self set diffFilter_ $val}
+
 Simulator instproc MPLS { val } { 
 	if { $val == "ON" } {
 		Node enable-module "MPLS"
@@ -362,6 +388,7 @@ Simulator instproc MPLS { val } {
 	}
 }
 
+
 Simulator instproc PGM { val } { 
         if { $val == "ON" } {
                 Node enable-module "PGM"
@@ -416,9 +443,10 @@ Simulator instproc node-config args {
         set args [eval $self init-vars $args]
 
         $self instvar addressType_  routingAgent_ propType_  macTrace_ \
-		routerTrace_ agentTrace_ movementTrace_ channelType_ channel_ \
-		chan topoInstance_ propInstance_ mobileIP_ rxPower_ \
-		txPower_ idlePower_ satNodeType_
+	    routerTrace_ agentTrace_ movementTrace_ channelType_ channel_ \
+	    chan topoInstance_ propInstance_ mobileIP_ \
+	    rxPower_ txPower_ idlePower_ sleepPower_ sleepTime_ transitionPower_ \
+	    transitionTime_ satNodeType_ eotTrace_
 
         if [info exists macTrace_] {
 		Simulator set MacTrace_ $macTrace_
@@ -432,6 +460,12 @@ Simulator instproc node-config args {
         if [info exists movementTrace_] {
 		Simulator set MovementTrace_ $movementTrace_
 	}
+
+	# change wrt Mike's code
+	if [info exists eotTrace_] {
+                Simulator set EotTrace_ $eotTrace_
+        }
+
         # hacking for matching old cmu add-interface
         # not good style, for back-compability ONLY
 	#
@@ -497,18 +531,7 @@ Simulator instproc node args {
                       Setting this variable will not affect simulations."
                 Simulator unset NumberInterfaces_
         }
-	
-	# wireless-ready node
-	if { [info exists routingAgent_] && ($routingAgent_ != "") } {
-		set node [eval $self create-wireless-node $args]
-		# for base node
-		if {[info exists wiredRouting_] && $wiredRouting_ == "ON"} {
-			set Node_([$node id]) $node
-			#simulator's nodelist in C++ space
-			$self add-node $node [$node id] 
-		}
-		return $node
-	}
+
 	# Satellite node
 	if { [info exists satNodeType_] } {
 		set node [eval $self create-satnode]
@@ -521,6 +544,19 @@ Simulator instproc node args {
 		}
 		return $node
 	}
+	
+	# wireless-ready node
+	if { [info exists routingAgent_] && ($routingAgent_ != "") } {
+		set node [eval $self create-wireless-node $args]
+		# for base node
+		if {[info exists wiredRouting_] && $wiredRouting_ == "ON"} {
+			set Node_([$node id]) $node
+			#simulator's nodelist in C++ space
+			$self add-node $node [$node id] 
+		}
+		return $node
+	}
+	
 
 	# Enable-mcast is now done automatically inside Node::init{}
 	# 
@@ -555,9 +591,10 @@ Simulator instproc imep-support {} {
 # of standing here in ns-lib.tcl.
 Simulator instproc create-wireless-node args {
         $self instvar routingAgent_ wiredRouting_ propInstance_ llType_ \
-		macType_ ifqType_ ifqlen_ phyType_ chan antType_ energyModel_ \
-		initialEnergy_ txPower_ rxPower_ idlePower_ \
-		topoInstance_ level1_ level2_ inerrProc_ outerrProc_ FECProc_
+	    macType_ ifqType_ ifqlen_ phyType_ chan antType_ \
+	    energyModel_ initialEnergy_ txPower_ rxPower_ \
+	    idlePower_ sleepPower_ sleepTime_ transitionPower_ transitionTime_ \
+	    topoInstance_ level1_ level2_ inerrProc_ outerrProc_ FECProc_
 
 	Simulator set IMEPFlag_ OFF
 
@@ -605,7 +642,11 @@ Simulator instproc create-wireless-node args {
 	    DumbAgent {
 		    set ragent [$self create-dumb-agent $node]
 	    }
+	    ManualRtg {
+		    set ragent [$self create-manual-rtg-agent $node]
+	    }
 	    default {
+		    eval $node addr $args
 		    puts "Wrong node routing agent!"
 		    exit
 	    }
@@ -623,9 +664,11 @@ Simulator instproc create-wireless-node args {
 		set FECProc_ ""
 	}
 
+	
+
 	# Add main node interface
 	$node add-interface $chan $propInstance_ $llType_ $macType_ \
-			$ifqType_ $ifqlen_ $phyType_ $antType_ \
+	    $ifqType_ $ifqlen_ $phyType_ $antType_ $topoInstance_ \
 			$inerrProc_ $outerrProc_ $FECProc_
 	# Attach agent
 	if {$routingAgent_ != "DSR"} {
@@ -688,7 +731,22 @@ Simulator instproc create-wireless-node args {
         if [info exists idlePower_] {
 		$node setPidle $idlePower_
         }
+#
+	if [info exists sleepPower_] {
+		$node setPsleep $sleepPower_
+        }
+	if [info exists sleepTime_] {
+		$node setTSleep $sleepTime_
+        }
+	if [info exists transitionPower_] {
+		$node setPtransition $transitionPower_
+        }
+	if [info exists transitionTime_] {
+		$node setTtransition $transitionTime_
+        }	
+#
 	$node topography $topoInstance_
+	
 	return $node
 }
 
@@ -754,6 +812,19 @@ Simulator instproc create-dumb-agent { node } {
 	return $ragent
 }
 
+Simulator instproc create-manual-rtg-agent { node } {
+	
+	# create a simple wireless agent
+	# that only forwards packets
+	# used for testing single hop brdcast/unicast mode 
+	# for wireless macs
+
+	set ragent [new Agent/ManualRtgAgent]
+	$node set ragent_ $ragent
+	$node attach $ragent [Node set rtagent_port_]
+	
+	return $ragent
+}
 
 Simulator instproc create-aodv-agent { node } {
         #  Create AODV routing agent
@@ -933,24 +1004,24 @@ Simulator instproc simplex-link { n1 n2 bw delay qtype args } {
 	# construct the queue
 	set qtypeOrig $qtype
 	switch -exact $qtype {
-	    ErrorModule {
-		if { [llength $args] > 0 } {
-		    set q [eval new $qtype $args]
-		} else {
-		    set q [new $qtype Fid]
+		ErrorModule {
+			if { [llength $args] > 0 } {
+				set q [eval new $qtype $args]
+			} else {
+				set q [new $qtype Fid]
+			}
 		}
-	    }
-	    intserv {
-		set qtype [lindex $args 0]
-		set q [new Queue/$qtype]
-	    }
-	    default {
-		if { [llength $args] == 0} {
-		    set q [new Queue/$qtype]
-		} else {
-		    set q [new Queue/$qtype $args]
+		intserv {
+			set qtype [lindex $args 0]
+			set q [new Queue/$qtype]
+		}
+		default {
+			if { [llength $args] == 0} {
+				set q [new Queue/$qtype]
+			} else {
+				set q [new Queue/$qtype $args]
+			}
 		}
-	    }
 	}
 	# Now create the link
 	switch -exact $qtypeOrig {
@@ -994,11 +1065,12 @@ Simulator instproc simplex-link { n1 n2 bw delay qtype args } {
 	
 	#XXX yuck
 	if {[string first "RED" $qtype] != -1 || 
-		[string first "PI" $qtype] != -1 || 
-		[string first "Vq" $qtype] != -1 ||
-		[string first "REM" $qtype] != -1 ||  
-		[string first "GK" $qtype] != -1 ||  
-		[string first "RIO" $qtype] != -1} {
+	    [string first "PI" $qtype] != -1 || 
+	    [string first "Vq" $qtype] != -1 ||
+	    [string first "REM" $qtype] != -1 ||  
+	    [string first "GK" $qtype] != -1 ||  
+	    [string first "RIO" $qtype] != -1 ||
+	    [string first "XCP" $qtype] != -1} {
 		$q link [$link_($sid:$did) set link_]
 	}
 
@@ -1077,6 +1149,47 @@ Simulator instproc remove-nam-linkconfig {i1 i2} {
 	}
 }
 
+# Armando L. Caro Jr. <acaro@@cis,udel,edu> 10/22/2001
+#
+# we create a simplex link (NOT duplex) from the core to the interface. we can
+# use arbitrary params (bw, delay, etc) since we'll never actually transmit
+# data on these links. they are only used for routing (ie, to determine which 
+# interface a packet should go out from)
+#
+Simulator instproc multihome-add-interface { core if } {
+  	$self instvar link_
+  	set coreId [$core id]
+  	set ifId [$if id]
+
+	# arbitrary values (doesn't matter since link will NEVER be used!)
+	set bw 1Mb
+	set delay 100ms
+	set type DropTail
+
+	if [info exists link_($coreId:$ifId)] {
+		$self remove-nam-linkconfig $coreId $ifId
+	}
+	eval $self simplex-link $core $if $bw $delay $type 
+	# Modified by GFR for nix-vector routing
+	if { [Simulator set nix-routing] } {
+		# Inform nodes of neighbors
+		$n1 set-neighbor [$core id]
+		$n2 set-neighbor [$if id]
+	}
+
+
+    	$core instvar multihome_interfaces_ num_interfaces_
+    	set interface_ {}
+
+    	# interface node
+    	lappend interface_ $if
+
+    	# link from interface node to core node
+    	lappend interface_ [$link_($coreId:$ifId) set head_]
+
+    	lappend multihome_interfaces_ $interface_
+}
+
 Simulator instproc duplex-link { n1 n2 bw delay type args } {
 	$self instvar link_
 	set i1 [$n1 id]
@@ -1360,6 +1473,9 @@ Simulator instproc monitor-queue { n1 n2 qtrace { sampleInterval 0.1 } } {
 Simulator instproc queue-limit { n1 n2 limit } {
 	$self instvar link_
 	[$link_([$n1 id]:[$n2 id]) queue] set limit_ $limit
+	if {[[$link_([$n1 id]:[$n2 id]) queue] info class] == "Queue/XCP"} {
+		[$link_([$n1 id]:[$n2 id]) queue] queue-limit $limit
+	}
 }
 
 Simulator instproc drop-trace { n1 n2 trace } {
@@ -1372,9 +1488,50 @@ Simulator instproc cost {n1 n2 c} {
 	$link_([$n1 id]:[$n2 id]) cost $c
 }
 
+# Armando L. Caro Jr. <acaro@@cis,udel,edu> 10/22/2001
+Simulator instproc multihome-attach-agent { core agent } {
+      	$agent set-multihome-core [$core entry]
+
+      	foreach interface [$core set multihome_interfaces_] {
+  		set ifNode [lindex $interface 0]
+		set coreLink [lindex $interface 1]
+
+      		# attach agent to the node for each interface
+      		$ifNode attach $agent
+      		set addr [$agent set agent_addr_]
+      		set port [$agent set agent_port_]
+      		set entry [$ifNode entry]
+
+      		# give the interface info to the agent
+		$agent add-multihome-interface $addr $port $entry $coreLink
+			
+      		$agent instvar multihome_bindings_
+      		set binding_ {}
+      		lappend binding_ $addr
+      		lappend binding_ $port
+      		lappend multihome_bindings_ $binding_
+      	}
+}
+
 Simulator instproc attach-agent { node agent } {
 	$node attach $agent
 	# $agent set nodeid_ [$node id]
+
+        # Armando L. Caro Jr. <acaro@@cis,udel,edu> 10/22/2001 
+	#
+	# list of tuples (addr, port)
+	# This is NEEDED so that single homed agents can play with multihomed
+	# ones!
+	# multihoming only for SCTP agents -Padma H.
+	if {[lindex [split [$agent info class] "/"] 1] == "SCTP"} {
+		$agent instvar multihome_bindings_
+		set binding_ {}
+		set addr [$agent set agent_addr_]
+		set port [$agent set agent_port_]
+		lappend binding_ $addr
+		lappend binding_ $port
+		lappend multihome_bindings_ $binding_
+	}
 }
 
 Simulator instproc attach-tbf-agent { node agent tbf } {
@@ -1451,6 +1608,13 @@ Simulator instproc connect {src dst} {
 
 	$self instvar conn_ nconn_ sflows_ nsflows_ useasim_
 
+        # Armando L. Caro Jr. <acaro@@cis,udel,edu>
+	# does the agent type support multihoming??
+	# @@@ do we need to worry about $useasim_ below?? (wasn't in 2.1b8)
+    	if {[lindex [split [$src info class] "/"] 1] == "SCTP"} {
+    		$self multihome-connect $src $dst
+    	}
+
 	$self simplex-connect $src $dst
 	$self simplex-connect $dst $src
 
@@ -1474,6 +1638,38 @@ Simulator instproc connect {src dst} {
 	return $src
 }
 
+# Armando L. Caro Jr. <acaro@@cis,udel,edu> 10/12/2001
+Simulator instproc multihome-connect {src dst} {
+	
+        set destNum 0
+	foreach binding [$src set multihome_bindings_] {
+		incr destNum
+  		set addr [lindex $binding 0]
+  		set port [lindex $binding 1]
+      		$dst add-multihome-destination $addr $port
+    	}
+	if {$destNum == 0} {
+	        # src isn't multihomed, so make sure we do an
+		# add-multihome-destination
+		$dst add-multihome-destination \
+				[$src set agent_addr_] [$src set agent_port_]
+	}
+	
+        set destNum 0
+	foreach binding [$dst set multihome_bindings_] {
+		incr destNum
+  		set addr [lindex $binding 0]
+  		set port [lindex $binding 1]
+      		$src add-multihome-destination $addr $port
+    	}
+	if {$destNum == 0} {
+	        # dst isn't multihomed, so make sure we do an
+		# add-multihome-destination
+		$src add-multihome-destination \
+				[$dst set agent_addr_] [$dst set agent_port_]
+	}
+}
+
 Simulator instproc simplex-connect { src dst } {
 	$src set dst_addr_ [$dst set agent_addr_] 
 	$src set dst_port_ [$dst set agent_port_]
@@ -1654,8 +1850,8 @@ Classifier instproc adjacents {} {
 Classifier instproc in-slot? slot {
 	$self instvar slots_
 	set ret ""
-	if {[array size slots_] < $slot} {
-		set ret slots_($slot)
+	if {[info exists slots_($slot)]} {
+	        set ret $slots_($slot)
 	}
 	set ret
 }
@@ -1985,7 +2181,7 @@ Simulator instproc create-diffusion-probability-agent {node} {
 # Diffusioncore agent (in diffusion) maps to the wireless routing agent
 # in ns
 Simulator instproc create-core-diffusion-rtg-agent {node} {
-	$self instvar stopTime_
+	$self instvar stopTime_ diffFilter_
 	Node instvar ragent_ dmux_
 	set ragent [new Agent/DiffusionRouting [$node id]]
 	$node set ragent_ $ragent
@@ -1994,8 +2190,11 @@ Simulator instproc create-core-diffusion-rtg-agent {node} {
 	if { [info exists stopTime_] } {
 		$ragent stop-time $stopTime_
 	}
-	
-	$node create-diffusionApp-agent
+	if { ![info exists diffFilter_] } {
+		puts stderr "Error: No filter defined for diffusion!\n"
+		exit 1
+	}
+	$node create-diffusionApp-agent $diffFilter_
 	return $ragent
 }
 
