@@ -30,7 +30,7 @@
 # OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 # SUCH DAMAGE.
 #
-# @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/tcl/test/test-suite-red.tcl,v 1.51 2003/01/19 03:54:03 sfloyd Exp $
+# @(#) $Header: /cvsroot/nsnam/ns-2/tcl/test/test-suite-red.tcl,v 1.62 2006/12/21 20:01:41 tom_henderson Exp $
 #
 # This test suite reproduces most of the tests from the following note:
 # Floyd, S., 
@@ -42,11 +42,22 @@
 set dir [pwd]
 catch "cd tcl/test"
 source misc_simple.tcl
+remove-all-packet-headers       ; # removes all except common
+add-packet-header Flags IP RTP TCP  ; # hdrs reqd for validation test
+ 
+# FOR UPDATING GLOBAL DEFAULTS:
+Agent/TCP set precisionReduce_ false ;   # default changed on 2006/1/24.
+Agent/TCP set rtxcur_init_ 6.0 ;      # Default changed on 2006/01/21
+Agent/TCP set updated_rttvar_ false ;  # Variable added on 2006/1/21
 Agent/TCP set tcpTick_ 0.1
 # The default for tcpTick_ is being changed to reflect a changing reality.
 Agent/TCP set rfc2988_ false
 # The default for rfc2988_ is being changed to true.
 # FOR UPDATING GLOBAL DEFAULTS:
+Queue/RED set bytes_ false              
+# default changed on 10/11/2004.
+Queue/RED set queue_in_bytes_ false
+# default changed on 10/11/2004.
 Queue/RED set q_weight_ 0.002
 Queue/RED set thresh_ 5 
 Queue/RED set maxthresh_ 15
@@ -57,6 +68,9 @@ Agent/TCP set windowInit_ 1
 # The default is being changed to 2.
 Agent/TCP set singledup_ 0
 # The default is being changed to 1
+Queue/RED set use_mark_p_ false
+# The default is being changed to true.
+
 catch "cd $dir"
 #Agent/TCP set oldCode_ true
 Agent/TCP set minrto_ 0
@@ -64,6 +78,8 @@ Agent/TCP set minrto_ 0
 Agent/TCP set syn_ false
 Agent/TCP set delay_growth_ false
 # In preparation for changing the default values for syn_ and delay_growth_.
+Agent/TCP set SetCWRonRetransmit_ true
+# Changing the default value.
 
 set flowfile fairflow.tr; # file where flow data is written
 set flowgraphfile fairflow.xgr; # file given to graph tool 
@@ -566,7 +582,7 @@ TestSuite instproc create_flow_graph { graphtitle graphfile } {
         	puts "writing flow xgraph data to $graphfile..."
 	}
 
-        exec sort -n +1 +0 -o $flowfile $flowfile
+        exec sort -n -k2 -k1 -o $flowfile $flowfile
         exec awk [$self $awkprocedure_] $flowfile >@ $outdesc
         close $outdesc
 }
@@ -935,4 +951,57 @@ Test/q_weight_auto instproc init {} {
 #     $self next
 # }
 
+Class Test/congested -superclass TestSuite
+Test/congested instproc init {} {
+    $self instvar net_ test_
+    set net_ net3 
+    set test_ congested
+    Queue/RED set mark_p_ 0.1
+    Queue/RED set use_mark_p_ false
+    $self next
+}
+Test/congested instproc run {} {
+    $self instvar ns_ node_ testName_ net_
+    Agent/TCP set packetSize_ 1500
+    Agent/TCP set window_ 75
+    Agent/TCP set ecn_ 1
+    Queue/RED set bytes_ true
+    Queue/RED set gentle_ false
+    Queue/RED set setbit_ true
+    $self setTopo
+    # The default is being changed to true.
+
+    set stoptime 5.0
+    set slink [$ns_ link $node_(r1) $node_(r2)]; # link to collect stats on
+    set fmon [$ns_ makeflowmon Fid]
+   #$ns_ attach-fmon $slink $fmon
+    $ns_ attach-fmon $slink $fmon 1
+    
+    set tcp1 [$ns_ create-connection TCP/Sack1 $node_(s1) TCPSink/Sack1 $node_(s3) 0]
+
+    set ftp1 [$tcp1 attach-app FTP]
+
+    $self enable_tracequeue $ns_
+    $ns_ at 0.0 "$ftp1 start"
+    $ns_ at $stoptime "$self printall $fmon"
+
+    $self tcpDump $tcp1 5.0
+
+    # trace only the bottleneck link
+    #$self traceQueues $node_(r1) [$self openTrace $stoptime $testName_]
+    $ns_ at $stoptime "$self cleanupAll $testName_"
+
+    $ns_ run
+}
+
+Class Test/congested_mark_p -superclass TestSuite
+Test/congested_mark_p instproc init {} {
+    $self instvar net_ test_
+    set net_ net3 
+    set test_ congested_mark_p
+    Queue/RED set mark_p_ 1.0
+    Queue/RED set use_mark_p_ true
+    Test/congested_mark_p instproc run {} [Test/congested info instbody run ]
+    $self next
+}
 TestSuite runTest
