@@ -30,7 +30,7 @@
 # OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 # SUCH DAMAGE.
 #
-# @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/tcl/test/test-suite-simple.tcl,v 1.28 2002/10/23 23:20:40 sfloyd Exp $
+# @(#) $Header: /cvsroot/nsnam/ns-2/tcl/test/test-suite-simple.tcl,v 1.44 2006/01/25 22:02:05 sallyfloyd Exp $
 #
 #
 # This test suite reproduces most of the tests from the following note:
@@ -52,7 +52,28 @@
 
 # ns-random 0
 
+#source misc_simple.tcl
+#source support.tcl
+
+remove-all-packet-headers       ; # removes all except common
+add-packet-header Flags IP TCP TFRC TFRC_ACK ; # hdrs reqd for TCP
+
 # FOR UPDATING GLOBAL DEFAULTS:
+Agent/TCP set rtxcur_init_ 6.0 ;      # Default changed on 2006/01/21
+Agent/TCP set updated_rttvar_ false ;  # Variable added on 2006/1/21
+Agent/TFRC set ss_changes_ 1 ; 	# Added on 10/21/2004
+Agent/TFRC set slow_increase_ 1 ; 	# Added on 10/20/2004
+Agent/TFRC set rate_init_ 2 ;          # Added on 10/20/2004
+Agent/TFRC set rate_init_option_ 2 ;    # Added on 10/20/2004
+Agent/TFRC set useHeaders_ false ;     # Added on 6/24/2004, default of true
+Agent/TFRC set headersize_ 40 ;         # Changed on 6/24/2004 to 32.
+
+Agent/TCP set minrto_ 1
+# default changed on 10/14/2004.
+Queue/RED set bytes_ false              
+# default changed on 10/11/2004.
+Queue/RED set queue_in_bytes_ false
+# default changed on 10/11/2004.
 Queue/RED set q_weight_ 0.002
 Queue/RED set thresh_ 5 
 Queue/RED set maxthresh_ 15
@@ -119,10 +140,10 @@ TestSuite instproc finish file {
 	set perlCode {
 		sub BEGIN { $c = 0; @p = @a = @d = @lu = @ld = (); }
 		/^[\+-] / && do {
-			if ($F[4] eq 'tcp') {
+			if ($F[4] eq 'tcp' || $F[4] eq 'tcpFriend') {
  				push(@p, $F[1], ' ',		\
 					$F[7] + ($F[10] % 90) * 0.01, "\n");
-			} elsif ($F[4] eq 'ack') {
+			} elsif ($F[4] eq 'ack' || $F[4] eq 'tcpFriendCtl') {
  				push(@a, $F[1], ' ',		\
 					$F[7] + ($F[10] % 90) * 0.01, "\n");
 			}
@@ -502,7 +523,7 @@ Test/tahoe1 instproc init topo {
 Test/tahoe1 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	# Set up TCP connection
 	set tcp1 [$ns_ create-connection TCP $node_(s1) TCPSink $node_(k1) 0]
@@ -584,7 +605,7 @@ Test/tahoe2 instproc init topo {
 Test/tahoe2 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	set tcp1 [$ns_ create-connection TCP $node_(s1) TCPSink $node_(k1) 0]
 	$tcp1 set window_ 14
@@ -612,7 +633,7 @@ Test/tahoe3 instproc init topo {
 Test/tahoe3 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ queue-limit $node_(r1) $node_(k1) 8   
 	$ns_ queue-limit $node_(k1) $node_(r1) 8   
@@ -661,7 +682,7 @@ Test/tahoe4 instproc init topo {
 Test/tahoe4 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ delay $node_(s2) $node_(r1) 200ms
 	$ns_ delay $node_(r1) $node_(s2) 200ms
@@ -698,7 +719,7 @@ Test/no_bug instproc init topo {
 Test/no_bug instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ delay $node_(s1) $node_(r1) 3ms
 	$ns_ delay $node_(r1) $node_(s1) 3ms
@@ -733,7 +754,7 @@ Test/bug instproc init topo {
 Test/bug instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ delay $node_(s1) $node_(r1) 3ms
 	$ns_ delay $node_(r1) $node_(s1) 3ms
@@ -771,7 +792,7 @@ Test/reno1 instproc init topo {
 Test/reno1 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	set tcp1 [$ns_ create-connection TCP/Reno $node_(s1) TCPSink $node_(k1) 0]
 	$tcp1 set window_ 14
@@ -799,7 +820,7 @@ Test/reno instproc init topo {
 Test/reno instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	set tcp1 [$ns_ create-connection TCP/Reno $node_(s1) TCPSink $node_(k1) 0]
 	$tcp1 set window_ 28
@@ -828,7 +849,7 @@ Test/renoA instproc init topo {
 Test/renoA instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ queue-limit $node_(r1) $node_(k1) 8
 
@@ -868,7 +889,7 @@ Test/reno2 instproc init topo {
 Test/reno2 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ queue-limit $node_(r1) $node_(k1) 9
 
@@ -903,7 +924,7 @@ Test/reno3 instproc init topo {
 Test/reno3 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ queue-limit $node_(r1) $node_(k1) 8
 	$ns_ queue-limit $node_(k1) $node_(r1) 8
@@ -939,7 +960,7 @@ Test/reno4 instproc init topo {
 Test/reno4 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ queue-limit $node_(r1) $node_(r2) 29
 
@@ -970,7 +991,7 @@ Test/reno4a instproc init topo {
 Test/reno4a instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ queue-limit $node_(r1) $node_(r2) 29
 
@@ -996,21 +1017,20 @@ Test/reno5 instproc init topo {
 	set test_	reno5
 	set guide_	\
 	"Reno TCP, TCP/bugFix_ set to false."
+	Agent/TCP set bugFix_ false
 	$self next
 }
 Test/reno5 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ queue-limit $node_(r1) $node_(k1) 9
 
 	set tcp1 [$ns_ create-connection TCP/Reno $node_(s1) TCPSink $node_(k1) 0]
 	$tcp1 set window_ 50
-	$tcp1 set bugFix_ false
 	set tcp2 [$ns_ create-connection TCP/Reno $node_(s2) TCPSink $node_(k1) 1]
 	$tcp2 set window_ 20
-	$tcp2 set bugFix_ false
 
 	set ftp1 [$tcp1 attach-app FTP]
 	set ftp2 [$tcp2 attach-app FTP]
@@ -1025,6 +1045,20 @@ Test/reno5 instproc run {} {
 	$ns_ run
 }
 
+Class Test/reno5_nobug -superclass TestSuite
+Test/reno5_nobug instproc init topo {
+	$self instvar net_ defNet_ test_ guide_
+	set net_	$topo
+	set defNet_	net0
+	set test_	reno5_nobug
+	set guide_	\
+	"Reno TCP, TCP/bugFix_ set to true."
+	Agent/TCP set bugFix_ true
+	Test/reno5_nobug instproc run {} [Test/reno5 info instbody run ]
+
+	$self next
+}
+
 Class Test/telnet -superclass TestSuite
 Test/telnet instproc init topo {
 	$self instvar net_ defNet_ test_ guide_
@@ -1040,7 +1074,7 @@ Test/telnet instproc init topo {
 Test/telnet instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ queue-limit $node_(r1) $node_(k1) 8
 	$ns_ queue-limit $node_(k1) $node_(r1) 8
@@ -1082,7 +1116,7 @@ Test/delayed instproc init topo {
 Test/delayed instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	set tcp1 [$ns_ create-connection TCP $node_(s1) TCPSink/DelAck $node_(k1) 0]
 	$tcp1 set window_ 50
@@ -1112,7 +1146,7 @@ Test/phase instproc init topo {
 Test/phase instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ delay $node_(s2) $node_(r1) 3ms
 	$ns_ delay $node_(r1) $node_(s2) 3ms
@@ -1149,7 +1183,7 @@ Test/phase1 instproc init topo {
 Test/phase1 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ delay $node_(s2) $node_(r1) 9.5ms
 	$ns_ delay $node_(r1) $node_(s2) 9.5ms
@@ -1187,7 +1221,7 @@ Test/phase2 instproc init topo {
 Test/phase2 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ delay $node_(s2) $node_(r1) 3ms
 	$ns_ delay $node_(r1) $node_(s2) 3ms
@@ -1272,7 +1306,7 @@ Test/manyflows instproc init topo {
 Test/manyflows instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	# Set up TCP connections
 
@@ -1309,6 +1343,10 @@ TestSuite instproc printpkts { label tcp } {
         set rtt [expr $numRtts * $tcpTick_]
 	puts "tcp $label most_recent_rtt [format "%5.3f" $rtt]"  
 }
+TestSuite instproc printpktsTFRC { label tfrc } {
+        global tcpTick_ 
+	puts "tfrc $label data_pkts_sent [$tfrc set ndatapack_]"
+}
 TestSuite instproc printdrops { fid fmon } {
 	set fcl [$fmon classifier]; # flow classifier
 	#
@@ -1336,6 +1374,33 @@ TestSuite instproc printall { fmon } {
 	puts "aggregate per-link total_marks [$fmon set pmarks_]"
 	puts "aggregate per-link total_packets [$fmon set pdepartures_]"
 }
+TestSuite instproc printPeakRates { fmon time } {
+        $self instvar pastbytes
+        set allflows [$fmon flows]
+        foreach f $allflows {
+                set fid [$f set flowid_]
+		#set src [$f set src_]
+		#set dst [$f set dst_]
+                set bytes [$f set bdepartures_]
+                if [info exists pastbytes($f)] {
+                        set newbytes [expr $bytes - $pastbytes($f)]
+                }  else {
+                        set newbytes $bytes 
+                }
+                if {$newbytes > 0} {
+			#src: $src d st: $dst
+                        puts "time: [format "%3.2f" $time] fid: $fid new_bytes $newbytes"
+                        set pastbytes($f) $bytes
+                }
+        }
+}
+TestSuite instproc printPeakRate { fmon time interval } {
+        $self instvar ns_
+        set newTime [expr [$ns_ now] + $interval]
+        $ns_ at $time "$self printPeakRates $fmon $time"
+        $ns_ at $newTime "$self printPeakRate $fmon $newTime $interval"
+}
+
 
 Class Test/stats -superclass TestSuite
 Test/stats instproc init topo {
@@ -1350,7 +1415,7 @@ Test/stats instproc init topo {
 Test/stats instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_ 
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ delay $node_(s2) $node_(r1) 200ms
 	$ns_ delay $node_(r1) $node_(s2) 200ms
@@ -1405,7 +1470,7 @@ Test/statsECN instproc init topo {
 Test/statsECN instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_ 
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ delay $node_(s2) $node_(r1) 200ms
 	$ns_ delay $node_(r1) $node_(s2) 200ms
@@ -1453,7 +1518,7 @@ Test/stats1 instproc init topo {
 Test/stats1 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_ 
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 
 	$ns_ delay $node_(s2) $node_(r1) 200ms
 	$ns_ delay $node_(r1) $node_(s2) 200ms
@@ -1571,7 +1636,7 @@ Test/stats2 instproc init topo {
 Test/stats2 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_ 
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 	set stoptime 10.1 
 
 	set tcp0 [$ns_ create-connection TCP $node_(s1) TCPSink $node_(s3) 0]
@@ -1613,7 +1678,7 @@ Test/stats3 instproc init topo {
 Test/stats3 instproc run {} {
 	global quiet
 	$self instvar ns_ node_ testName_ guide_ 
-	if {$quiet == "false"} {puts $guide_}
+	puts "Guide: $guide_"
 	set stoptime 1.1 
 
 	set slink [$ns_ link $node_(r1) $node_(r2)]; 
@@ -1655,6 +1720,107 @@ Test/stats3 instproc run {} {
 	$ns_ run
 }
 
+Class Test/stats4 -superclass TestSuite
+Test/stats4 instproc init topo {
+	$self instvar net_ defNet_ test_ guide_
+	set net_	$topo
+	set defNet_	net3
+	set test_	stats4
+	Agent/TCP set tcpTick_ 0.01
+	set guide_	"Printing peak rate statistics."
+	$self next
+}
+
+Test/stats4 instproc run {} {
+	global quiet
+	$self instvar ns_ node_ testName_ guide_ 
+	puts "Guide: $guide_"
+	set stoptime 1.1 
+	set PeakRateInterval 0.1
+
+	set slink [$ns_ link $node_(r1) $node_(r2)]; 
+	#set fmon [$ns_ makeflowmon SrcDestFid]
+	set fmon [$ns_ makeflowmon Fid]
+	set outfile stdout
+	$fmon traceDist $outfile
+	$ns_ attach-fmon $slink $fmon
+
+	set tcp0 [$ns_ create-connection TCP $node_(s1) TCPSink $node_(s3) 0]
+	$tcp0 set window_ 10
+	set tcp1 [$ns_ create-connection TCP $node_(s2) TCPSink $node_(s3) 1]
+	$tcp1 set window_ 10
+
+	set ftp0 [$tcp0 attach-app FTP]
+	set ftp1 [$tcp1 attach-app FTP]
+	$ns_ at 0.0 "$ftp0 start"
+	$ns_ at 0.1 "$ftp1 start"
+
+	set link1 [$ns_ link $node_(r1) $node_(r2)]
+	set queue1 [$link1 queue]
+
+	$ns_ at 0.5 "$self printPeakRate $fmon 0.5 $PeakRateInterval"
+
+	# trace only the bottleneck link
+	$self traceQueues $node_(r1) [$self openTrace $stoptime $testName_]
+	$ns_ run
+}
+
+
+
+Class Test/statsTFRC -superclass TestSuite
+Test/statsTFRC instproc init topo {
+	$self instvar net_ defNet_ test_ guide_
+	set net_	$topo
+	set defNet_	net0
+	set test_	statsTFRC
+	set guide_	\
+	"TFRC statistics, and per-flow and aggregate link statistics."
+	$self next
+}
+Test/statsTFRC instproc run {} {
+	global quiet
+	$self instvar ns_ node_ testName_ guide_ test_
+	puts "Guide: $guide_"
+
+	$ns_ delay $node_(s2) $node_(r1) 200ms
+	$ns_ delay $node_(r1) $node_(s2) 200ms
+	$ns_ queue-limit $node_(r1) $node_(k1) 10
+	$ns_ queue-limit $node_(k1) $node_(r1) 10
+
+	set slink [$ns_ link $node_(r1) $node_(k1)]; # link to collect stats on
+	set fmon [$ns_ makeflowmon Fid]
+	$ns_ attach-fmon $slink $fmon
+
+	set stoptime 5.1 
+
+	set tf0 [$ns_ create-connection TFRC $node_(s1) TFRCSink $node_(k1) 0]
+	#$tf0 set window_ 30
+	set tf1 [$ns_ create-connection TFRC $node_(s2) TFRCSink $node_(k1) 1]
+	#$tf1 set window_ 30
+
+	set ftp0 [new Application/FTP]; $ftp0 attach-agent $tf0
+	set ftp1 [new Application/FTP]; $ftp1 attach-agent $tf1
+
+	$ns_ at 1.0 "$ftp0 start"
+	$ns_ at 1.0 "$ftp1 start"
+
+	#$self tcpDumpAll $tf0 5.0 tf0
+	#$self tcpDumpAll $tf1 5.00001 tf1
+
+	set almosttime [expr $stoptime - 0.001]
+	$ns_ at $almosttime "$self printpktsTFRC 0 $tf0"
+	$ns_ at $almosttime "$self printpktsTFRC 1 $tf1"
+	$ns_ at $stoptime "$self printdrops 0 $fmon; $self printdrops 1 $fmon"
+	$ns_ at $stoptime "$self printall $fmon"
+
+	# trace only the bottleneck link
+	$self traceQueues $node_(r1) [$self openTrace $stoptime $testName_]
+	$ns_ run
+}
+
+# Add a test to printTimestamps.
+# tcp-sink.cc, p. 337.
+
 TestSuite runTest
 
 ### Local Variables:
