@@ -1,5 +1,4 @@
-/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
-/*
+/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */ /*
  * Copyright (c) 1991-1997 Regents of the University of California.
  * All rights reserved.
  *
@@ -31,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/tcp/tcp.h,v 1.107 2003/02/12 04:16:10 sfloyd Exp $ (LBL)
+ * @(#) $Header: /cvsroot/nsnam/ns-2/tcp/tcp.h,v 1.126 2006/06/14 01:12:28 sallyfloyd Exp $ (LBL)
  */
 #ifndef ns_tcp_h
 #define ns_tcp_h
@@ -88,6 +87,8 @@ struct hdr_tcp {
 #define	CWND_ACTION_DUPACK	1	// dup acks/fast retransmit
 #define	CWND_ACTION_TIMEOUT	2	// retransmission timeout
 #define	CWND_ACTION_ECN		3	// ECN bit [src quench if supported]
+#define CWND_ACTION_EXITED      4       // congestion recovery has ended
+					// (when previously CWND_ACTION_DUPACK)
 
 /* these are bits for how to change the cwnd and ssthresh values */
 
@@ -101,6 +102,8 @@ struct hdr_tcp {
 #define THREE_QUARTER_SSTHRESH  0x00000080
 #define CLOSE_CWND_HALF_WAY 	0x00000100
 #define CWND_HALF_WITH_MIN	0x00000200
+#define TCP_IDLE		0x00000400
+#define NO_OUTSTANDING_DATA     0x00000800
 
 /*
  * tcp_tick_:
@@ -147,10 +150,32 @@ protected:
 	TcpAgent *a_;
 };
 
+/*
+ * Variables for HighSpeed TCP.
+ */
+//int *hs_win_;		// array of cwnd values
+//int *hs_increase_;	// array of increase values
+//double *hs_decrease_;	// array of decrease values
+struct hstcp {
+	double low_p;  // low_p
+	double dec1;	// for computing the decrease parameter
+	double dec2;    // for computing the decrease parameter
+	double p1;	// for computing p
+        double p2;	// for computing p
+	/* The next three parameters are for CPU overhead, for computing */
+	/*   the HighSpeed parameters less frequently.  A better solution */
+ 	/*   might be just to have a look-up array.  */
+	double cwnd_last_;	/* last cwnd for computed parameters */
+        double increase_last_;	/* increase param for cwnd_last_ */
+	hstcp() : low_p(0.0), dec1(0.0), dec2(0.0), p1(0.0), p2(0.0),
+	    cwnd_last_(0.0), increase_last_(0.0) { }
+};
 
 class TcpAgent : public Agent {
+	friend class XcpEndsys;
 public:
 	TcpAgent();
+	virtual ~TcpAgent() {free(tss);}
         virtual void recv(Packet*, Handler*);
 	virtual void timeout(int tno);
 	virtual void timeout_nonrtx(int tno);
@@ -170,30 +195,38 @@ protected:
 	virtual void delay_bind_init_all();
 	virtual int delay_bind_dispatch(const char *varName, const char *localName, TclObject *tracer);
 
+	TracedInt t_seqno_;	/* sequence number */
 	/*
 	 * State encompassing the round-trip-time estimate.
 	 * srtt and rttvar are stored as fixed point;
 	 * srtt has 3 bits to the right of the binary point, rttvar has 2.
 	 */
-	TracedInt t_seqno_;	/* sequence number */
-#define T_RTT_BITS 0
 	TracedInt t_rtt_;      	/* round trip time */
-	int T_SRTT_BITS;        /* exponent of weight for updating t_srtt_ */
 	TracedInt t_srtt_;     	/* smoothed round-trip time */
+	TracedInt t_rttvar_;   	/* variance in round-trip time */
+	TracedInt t_backoff_;	/* current multiplier, 1 if not backed off */
+	#define T_RTT_BITS 0
+	int T_SRTT_BITS;        /* exponent of weight for updating t_srtt_ */
 	int srtt_init_;		/* initial value for computing t_srtt_ */
 	int T_RTTVAR_BITS;      /* exponent of weight for updating t_rttvar_ */ 
 	int rttvar_exp_;        /* exponent of multiple for t_rtxcur_ */
-	TracedInt t_rttvar_;   	/* variance in round-trip time */
 	int rttvar_init_;       /* initial value for computing t_rttvar_ */
 	double t_rtxcur_;	/* current retransmit value */
 	double rtxcur_init_;    /* initial value for t_rtxcur_ */
-	TracedInt t_backoff_;	/* current multiplier, 1 if not backed off */
 	virtual void rtt_init();
 	virtual double rtt_timeout();	/* provide RTO based on RTT estimates */
 	virtual void rtt_update(double tao);	/* update RTT estimate */
 	virtual void rtt_backoff();		/* double multiplier */
+	/* End of state for the round-trip-time estimate. */
 
+	/* Timestamps. */
 	double ts_peer_;        /* the most recent timestamp the peer sent */
+	double ts_echo_;        /* the most recent timestamp the peer echoed */
+	int ts_option_size_;    // header bytes in a ts option
+        double *tss;            // To store sent timestamps, with bugfix_ts_
+        int tss_size_;          // Current capacity of tss
+	int ts_option_;		/* use RFC1323-like timestamps? */
+	/* End of timestamps. */
 
 	/* connection and packet dynamics */
 	virtual void output(int seqno, int reason = 0);
@@ -203,7 +236,7 @@ protected:
 	virtual void send_one();		/* do this on 1-2 dupacks */
 	double linear(double x, double x_1, double y_1, double x_2, double y_2);
 	/* the "linear" function is for experimental highspeed TCP */
-	void opencwnd();
+	virtual void opencwnd();
 
 	void slowdown(int how);			/* reduce cwnd/ssthresh */
 	void ecn(int seqno);		/* react to quench */
@@ -216,19 +249,25 @@ protected:
 	void reset_qoption();	/* for QOption with EnblRTTCtr_ */
 	void rtt_counting();	/* for QOption with EnblRTTCtr_ */
 	int network_limited();	/* Sending limited by network? */
-	double limited_slow_start(double cwnd, double max_ssthresh, double increment);
+	double limited_slow_start(double cwnd, int max_ssthresh, double increment);
 				/* Limited slow-start for high windows */
 	virtual int numdupacks(double cwnd); 	/* for getting numdupacks_ */
 	virtual void processQuickStart(Packet *pkt);
 	virtual void endQuickStart();
+	int lossQuickStart();
 
 	/* Helper functions. Currently used by tcp-asym */
 	virtual void output_helper(Packet*) { return; }
 	virtual void send_helper(int) { return; }
 	virtual void send_idle_helper() { return; }
 	virtual void recv_helper(Packet*) { return; }
+	virtual void recv_frto_helper(Packet*);
 	virtual void recv_newack_helper(Packet*);
 	virtual void partialnewack_helper(Packet*) {};
+	/* End of helper functions. */
+
+	int force_wnd(int num);
+	void spurious_timeout();
 
 	/* Timers */
 	RtxTimer rtx_timer_;
@@ -247,6 +286,8 @@ protected:
 	int timerfix_;		/* set to true to update timer *after* */
 				/* update the RTT, instead of before   */
 	int rfc2988_;		/* Use updated RFC 2988 timers */
+	/* End of timers. */ 
+
 	double boot_time_;	/* where between 'ticks' this sytem came up */
 	double overhead_;
 	double wnd_;
@@ -260,72 +301,97 @@ protected:
 				/* 2 for using large initial windows */
 	double decrease_num_;   /* factor for multiplicative decrease */
 	double increase_num_;   /* factor for additive increase */
-	double k_parameter_;     /* k parameter in binomial controls */
-	double l_parameter_;     /* l parameter in binomial controls */
-	int precision_reduce_;  /* non-integer reduction of cwnd */
-	int syn_;		/* 1 for modeling SYN/ACK exchange */
-	int delay_growth_;  	/* delay opening cwnd until 1st data recv'd */
 	int tcpip_base_hdr_size_;  /* size of base TCP/IP header */
-	int ts_option_size_;    // header bytes in a ts option
-	int bug_fix_;		/* 1 for multiple-fast-retransmit fix */
-	int less_careful_;	/* 1 for Less Careful variant of bug_fix_, */
-				/*  for illustration only  */
-	int ts_option_;		/* use RFC1323-like timestamps? */
-	int maxburst_;		/* max # packets can send back-2-back */
+	int ts_resetRTO_;	/* Un-backoff RTO after any valid RTT, */
+				/*   including from a retransmitted pkt?  */
+				/* The old version was "false". */
+				/* But "true" gives better performance, and */
+                                /* seems conformant with RFC 2988. */
 	int maxcwnd_;		/* max # cwnd can ever be */
         int numdupacks_;	/* dup ACKs before fast retransmit */
 	int numdupacksFrac_;	/* for a larger numdupacks_ with large */
 				/* windows */
 	double maxrto_;		/* max value of an RTO */
 	double minrto_;         /* min value of an RTO */
+
+	/* For modeling SYN and SYN/ACK packets. */
+	int syn_;		/* 1 for modeling SYN/ACK exchange */
+	int delay_growth_;  	/* delay opening cwnd until 1st data recv'd */
+	/* End of modeling SYN and SYN/ACK packets. */
+
+	/* F-RTO */
+	int frto_enabled_;	/* != 0 to enable F-RTO */
+	int sfrto_enabled_;	/* != 0 to enabled SACK-based F-RTO */
+	int spurious_response_;	/* Response variant to spurious RTO */
+	/* End of R-RTO */
+
+	/* Parameters for backwards compatility with old code. */ 
+	int bug_fix_;		/* 1 for multiple-fast-retransmit fix */
+	int less_careful_;	/* 1 for Less Careful variant of bug_fix_, */
+				/*  for illustration only  */
+	int exitFastRetrans_;	/* True to clean exits of Fast Retransmit */ 
+				/* False for buggy old behavior */
+	int bugfix_ack_;        // 1 to enable ACK heuristic, to allow
+				//  multiple-fast-retransmits in special cases.
+				// From Andrei Gurtov
+	int bugfix_ts_;         // 1 to enable timestamp heuristic, to allow
+				//  multiple-fast-retransmits in special cases.
+				// From Andrei Gurtov
+				// Not implemented yet.
 	int old_ecn_;		/* For backwards compatibility with the 
 				 * old ECN implementation, which never
 				 * reduced the congestion window below
 				 * one packet. */ 
+	int bugfix_ss_;		// 1 to use window of one when SYN
+				//  packet is dropped
+	/* End of parameters for backwards compatility. */
+
+	/* Parameters for alternate congestion control mechanisms. */
+	double k_parameter_;     /* k parameter in binomial controls */
+	double l_parameter_;     /* l parameter in binomial controls */
+	int precision_reduce_;  /* non-integer reduction of cwnd */
+	int maxburst_;		/* max # packets can send back-2-back */
+	int aggressive_maxburst_;	/* Send on a non-valid ack? */
+	/* End of parameters for alternate congestion control mechanisms. */
+
 	FILE *plotfile_;
 	/*
 	 * Dynamic state.
 	 */
 	TracedInt dupacks_;	/* number of duplicate acks */
 	TracedInt curseq_;	/* highest seqno "produced by app" */
+	TracedInt highest_ack_;	/* not frozen during Fast Recovery */
+	TracedDouble cwnd_;	/* current window */
+	TracedInt ssthresh_;	/* slow start threshold */
+	TracedInt maxseq_;	/* used for Karn algorithm */
+				/* highest seqno sent so far */
 	int last_ack_;		/* largest consecutive ACK, frozen during
 				 *		Fast Recovery */
-	TracedInt highest_ack_;	/* not frozen during Fast Recovery */
 	int recover_;		/* highest pkt sent before dup acks, */
 				/*   timeout, or source quench/ecn */
 	int last_cwnd_action_;	/* CWND_ACTION_{TIMEOUT,DUPACK,ECN} */
-	TracedDouble cwnd_;	/* current window */
-	double base_cwnd_;	/* base window (for experimental purposes) */
-	double awnd_;		/* averaged window */
-	TracedInt ssthresh_;	/* slow start threshold */
 	int count_;		/* used in window increment algorithms */
-	double fcnt_;		/* used in window increment algorithms */
 	int rtt_active_;	/* 1 if a rtt sample is pending */
 	int rtt_seq_;		/* seq # of timed seg if rtt_active_ is 1 */
 	double rtt_ts_;		/* time at which rtt_seq_ was sent */
-	TracedInt maxseq_;	/* used for Karn algorithm */
-				/* highest seqno sent so far */
-	int ecn_;		/* Explicit Congestion Notification */
-	int cong_action_;	/* Congestion Action.  True to indicate
-				   that the sender responded to congestion. */
-        int ecn_burst_;		/* True when the previous ACK packet
-				 *  carried ECN-Echo. */
-	int ecn_backoff_;	/* True when retransmit timer should begin
-			  	    to be backed off.  */
-	int ect_;       	/* turn on ect bit now? */
-        int eln_;               /* Explicit Loss Notification (wireless) */
-        int eln_rxmit_thresh_;  /* Threshold for ELN-triggered rxmissions */
-        int eln_last_rxmit_;    /* Last packet rxmitted due to ELN info */
 	double firstsent_;	/* When first packet was sent  --Allman */
 	double lastreset_;	/* W.N. Last time connection was reset - for */
 				/* detecting pkts from previous incarnations */
-	int slow_start_restart_; /* boolean: re-init cwnd after connection 
-				    goes idle.  On by default. */
-	int restart_bugfix_;    /* ssthresh is cut down because of
-				   timeouts during a connection's idle period.
-				   Setting this boolean fixes this problem.
-				   For now, it is off by default. */ 
 	int closed_;            /* whether this connection has closed */
+	int use_rtt_;		/* Use RTT for timeout, for ECN-marked SYN-ACK */
+
+	/* Dynamic state used for alternate congestion control mechanisms */
+	double awnd_;		/* averaged window */
+	int first_decrease_;	/* First decrease of congestion window.  */
+				/* Used for decrease_num_ != 0.5. */
+	double fcnt_;		/* used in window increment algorithms */
+	double base_cwnd_;	/* base window (for experimental purposes) */
+	/* End of state for alternate congestion control mechanisms */
+
+	/* Dynamic state only used for monitoring */
+	int trace_all_oneline_;	/* TCP tracing vars all in one line or not? */
+	int nam_tracevar_;      /* Output nam's variable trace or just plain 
+				   text variable trace? */
         TracedInt ndatapack_;   /* number of data packets sent */
         TracedInt ndatabytes_;  /* number of data bytes sent */
         TracedInt nackpack_;    /* number of ack packets received */
@@ -337,15 +403,44 @@ protected:
 			   	   in response to an ecn packet -- sylvia */
         TracedInt ncwndcuts_; 	/* number of times cwnd was reduced 
 				   for any reason -- sylvia */
-	int trace_all_oneline_;	/* TCP tracing vars all in one line or not? */
-	int nam_tracevar_;      /* Output nam's variable trace or just plain 
-				   text variable trace? */
-	int first_decrease_;	/* First decrease of congestion window.  */
-				/* Used for decrease_num_ != 0.5. */
+        TracedInt ncwndcuts1_;     /* number of times cwnd was reduced 
+                                   due to congestion (as opposed to idle
+                                   periods */
+	/* end of dynamic state for monitoring */
+
+	/* Specifying variants in TCP algorithms.  */
+	int slow_start_restart_; /* boolean: re-init cwnd after connection 
+				    goes idle.  On by default. */
+	int restart_bugfix_;    /* ssthresh is cut down because of
+				   timeouts during a connection's idle period.
+				   Setting this boolean fixes this problem.
+				   For now, it is off by default. */ 
         TracedInt singledup_;   /* Send on a single dup ack.  */
+	int LimTransmitFix_;	/* To fix a bug in Limited Transmit. */
 	int noFastRetrans_;	/* No Fast Retransmit option.  */
 	int oldCode_;		/* Use old code. */
 	int useHeaders_;	/* boolean: Add TCP/IP header sizes */
+	/* end of specifying variants */
+
+	/* Used for ECN */
+	int ecn_;		/* Explicit Congestion Notification */
+	int cong_action_;	/* Congestion Action.  True to indicate
+				   that the sender responded to congestion. */
+        int ecn_burst_;		/* True when the previous ACK packet
+				 *  carried ECN-Echo. */
+	int ecn_backoff_;	/* True when retransmit timer should begin
+			  	    to be backed off.  */
+	int ect_;       	/* turn on ect bit now? */
+	int SetCWRonRetransmit_;  /* True to allow setting CWR on */
+				  /*  retransmitted packets.   Affects */
+				  /*  performance for Reno with ECN.  */
+	/* end of ECN */
+
+	/* used for Explicit Loss Notification */
+        int eln_;               /* Explicit Loss Notification (wireless) */
+        int eln_rxmit_thresh_;  /* Threshold for ELN-triggered rxmissions */
+        int eln_last_rxmit_;    /* Last packet rxmitted due to ELN info */
+	/* end of Explicit Loss Notification */
 
 	/* for experimental high-speed TCP */
 	/* These four parameters define the HighSpeed response function. */
@@ -356,26 +451,44 @@ protected:
 	/* The next parameter is for Limited Slow-Start. */
 	int max_ssthresh_;	/* max value for ssthresh_ */
 
-	/* These three functions are just an easy structuring of the code. */ 
+	/* These two functions are just an easy structuring of the code. */ 
 	double increase_param();  /* get increase parameter for current cwnd */
 	double decrease_param();  /* get decrease parameter for current cwnd */
-	double compute_p();	/* compute p for calculating parameters */
-	/* The next three parameters are for CPU overhead, for computing */
-	/*   the HighSpeed parameters less frequently.  A better solution */
- 	/*   might be just to have a look-up array.  */
-	double cwnd_last_;	/* last cwnd for computed parameters */
-        double increase_last_;	/* increase param for cwnd_last_ */
-	double cwnd_frac_;	/* for determining when to recompute params. */
+	int cwnd_range_;	/* for determining when to recompute params. */
+	hstcp hstcp_;		/* HighSpeed TCP variables */
         /* end of section for experimental high-speed TCP */
 
 	/* for Quick-Start, experimental. */
-	int rate_request_;	/* Rate request in pps, for QuickStart. */
-	int qs_enabled_; /* to enable QuickStart. */
+	int rate_request_;      /* Rate request in KBps, for QuickStart.  */
+	int qs_enabled_;        /* to enable QuickStart. */
 	int qs_requested_;
 	int qs_approved_;
+        int qs_window_;  /* >0: there are outstanding non-acked segments
+                            from QS window */
+        int qs_cwnd_; /* Initial window for Quick-Start */
+        int tcp_qs_recovery_; /* != 0 if we apply slow start on packet
+                                 losses during QS window */
+	int qs_request_mode_;   /* 1 = Try to avoid unnecessary QS requests
+				   for short flows. Use qs_rtt_ as the RTT
+				   used in window calculation.
+				   Other: Always request 'rate_request_' bytes,
+				   regardless of flow size */
+	int qs_thresh_;         /* Do not use QS if there are less data to send
+				   than this. Applies only if
+				   qs_request_mode_ == 1 */
+	int qs_rtt_;            /* QS needs some assumption of the RTT in
+				   in order to be able to determine how much
+				   it needs for rate request with given amount
+				   of data to send. milliseconds. */
+	int print_request_;	/* true to print Quick-Start request */
 	int ttl_diff_;
         /* end of section for Quick-Start. */
 
+	/* F-RTO: !=0 when F-RTO recovery is underway, N:th round-trip
+	 * since RTO. Can have values between 0-2 */
+	int frto_;
+	int pipe_prev_; /* window size when timeout last occurred */
+
         /* support for event-tracing */
         //EventTrace *et_;
         void trace_event(char *eventtype);
@@ -383,10 +496,10 @@ protected:
 	/* these function are now obsolete, see other above */
 	void closecwnd(int how);
 	void quench(int how);
-
+        
+	/* TCP quiescence, reducing cwnd after an idle period */
 	void process_qoption_after_send() ;
 	void process_qoption_after_ack(int seqno) ;
-
 	int QOption_ ; /* TCP quiescence option */
 	int EnblRTTCtr_ ; /* are we using a corase grained timer? */
 	int T_full ; /* last time the window was full */
@@ -401,11 +514,11 @@ protected:
 	int W_timed ;
 	int F_full ; 
 	int Backoffs ;
-
 	int control_increase_ ; /* If true, don't increase cwnd if sender */
 				/*  is not window-limited.  */
 	int prev_highest_ack_ ; /* Used to determine if sender is */
 				/*  window-limited.  */
+   	/* end of TCP quiescence */
 };
 
 /* TCP Reno */
@@ -523,6 +636,7 @@ protected:
 
 // Local Variables:
 // mode:c++
+// c-basic-offset: 8
 // End:
 
 #endif
