@@ -65,6 +65,8 @@ TfrcSinkAgent::TfrcSinkAgent() : Agent(PT_TFRC_ACK), nack_timer_(this)
 	bind ("NumSamples_", &numsamples);
 	bind ("discount_", &discount);
 	bind ("smooth_", &smooth_);
+	bind ("ShortIntervals_", &ShortIntervals_);
+	bind ("ShortRtts_", &ShortRtts_);
 
 	// EWMA use only
 	bind ("history_", &history); // EWMA history
@@ -72,11 +74,15 @@ TfrcSinkAgent::TfrcSinkAgent() : Agent(PT_TFRC_ACK), nack_timer_(this)
 	// for RBPH use only
 	bind("minlc_", &minlc); 
 
+	bind("bytes_", &bytes_);
 	rtt_ =  0; 
 	tzero_ = 0;
 	last_timestamp_ = 0;
 	last_arrival_ = 0;
 	last_report_sent=0;
+	total_received_ = 0;
+	total_losses_ = 0;
+	total_dropped_ = 0;
 
 	maxseq = -1;
 	maxseqList = -1;
@@ -99,6 +105,9 @@ TfrcSinkAgent::TfrcSinkAgent() : Agent(PT_TFRC_ACK), nack_timer_(this)
 	sample = NULL ; 
 	weights = NULL ;
 	mult = NULL ;
+        losses = NULL ;
+	count_losses = NULL ;
+        num_rtts = NULL ;
 	sample_count = 1 ;
 	mult_factor_ = 1.0;
 	init_WALI_flag = 0;
@@ -112,7 +121,8 @@ TfrcSinkAgent::TfrcSinkAgent() : Agent(PT_TFRC_ACK), nack_timer_(this)
 }
 
 /*
- * This is a new loss event if it is at least an RTT after the last one.
+ * This is a new loss event if it is at least an RTT after the beginning
+ *   of the last one.
  * If PreciseLoss_ is set, the new_loss also checks that there is a
  *     new round_id.
  * The sender updates the round_id when it receives a new report from
@@ -123,10 +133,19 @@ TfrcSinkAgent::TfrcSinkAgent() : Agent(PT_TFRC_ACK), nack_timer_(this)
  */
 int TfrcSinkAgent::new_loss(int i, double tstamp)
 {
-	if ((tsvec_[i%hsz]-lastloss > rtt_) 
+	double time_since_last_loss_interval = tsvec_[i%hsz]-lastloss;
+	if ((time_since_last_loss_interval > rtt_)
 	     && (PreciseLoss_ == 0 || (round_id > lastloss_round_id))) {
 		lastloss = tstamp;
 		lastloss_round_id = round_id ;
+                if (time_since_last_loss_interval < ShortRtts_ * rtt_ &&
+				algo == WALI) {
+                        count_losses[0] = 1;
+                }
+                if (rtt_ > 0 && algo == WALI) {
+                        num_rtts[0] = (int) ceil(time_since_last_loss_interval / rtt_);
+                        if (num_rtts[0] < 1) num_rtts[0] = 1;
+                }
 		return TRUE;
 	} else return FALSE;
 }
@@ -152,7 +171,13 @@ void TfrcSinkAgent::recv(Packet *pkt, Handler *)
 	int UrgentFlag = 0;	// send loss report immediately
 	int newdata = 0;	// a new data packet received
 
+	if (algo == WALI && !init_WALI_flag) {
+		init_WALI () ;
+	}
 	rcvd_since_last_report ++;
+	total_received_ ++;
+	// bytes_ was added by Tom Phelan, for reporting bytes received.
+	bytes_ += hdr_cmn::access(pkt)->size();
 
 	if (maxseq < 0) {
 		// This is the first data packet.
@@ -178,6 +203,7 @@ void TfrcSinkAgent::recv(Packet *pkt, Handler *)
 	/* for the time being, we will ignore out of order and duplicate 
 	   packets etc. */
 	int seqno = tfrch->seqno ;
+	fsize_ = tfrch->fsize;
 	int oldmaxseq = maxseq;
 	// if this is the highest packet yet, or an unknown packet
 	//   between maxseqList and maxseq  
@@ -196,18 +222,38 @@ void TfrcSinkAgent::recv(Packet *pkt, Handler *)
 		rtvec_[seqno%hsz]=now;	
 		tsvec_[seqno%hsz]=last_timestamp_;	
 		if (hf->ect() == 1 && hf->ce() == 1) {
+			// ECN action
 			lossvec_[seqno%hsz] = ECN_RCVD;
+			++ total_losses_;
+			losses_since_last_report++;
 			if (new_loss(seqno, tsvec_[seqno%hsz])) {
-				// ECN action
 				ecnEvent = 1;
 				lossvec_[seqno%hsz] = ECNLOST;
-				losses_since_last_report++;
 			} 
+			if (algo == WALI) {
+                       		++ losses[0];
+			}
 		} else lossvec_[seqno%hsz] = RCVD;
 	}
+	if (seqno > maxseq) {
+		int i = maxseq + 1;
+		while (i < seqno) {
+			// Added 3/1/05 in case we have wrapped around
+			//   in packet sequence space.
+			lossvec_[i%hsz] = UNKNOWN;
+			++ i;
+			++ total_losses_;
+			++ total_dropped_;
+		}
+	}
 	if (seqno > maxseqList && 
 	  (ecnEvent || numPktsSoFar_ >= numPkts_ ||
 	     tsvec_[seqno%hsz] - tsvec_[maxseqList%hsz] > rtt_)) {
+		// numPktsSoFar_ >= numPkts_:
+		// Number of pkts since we last entered this procedure
+		//   at least equal numPkts_, the number of non-sequential 
+		//   packets that must be seen before inferring loss.
+		// maxseqList: max seq number checked for dropped packets
 		// Decide which losses begin new loss events.
 		int i = maxseqList ;
 		while(i < seqno) {
@@ -222,6 +268,9 @@ void TfrcSinkAgent::recv(Packet *pkt, Handler *)
 					// as it does not begin a loss event.
 					lossvec_[i%hsz] = NOT_RCVD; 
 				}
+				if (algo == WALI) {
+			    		++ losses[0];
+				}
 				losses_since_last_report++;
 			}
 			i++;
@@ -281,7 +330,7 @@ double TfrcSinkAgent::est_loss ()
 }
 
 /*
- * compute estimated throughput for report.
+ * compute estimated throughput in packets per RTT for report.
  */
 double TfrcSinkAgent::est_thput () 
 {
@@ -292,7 +341,7 @@ double TfrcSinkAgent::est_thput ()
 	if ((rtt_ > 0) && ((now - last_report_sent) >= rtt_)) {
 		// more than an RTT since the last report
 		time_for_rcv_rate = (now - last_report_sent);
-		if (time_for_rcv_rate > 0 && rcvd_since_last_report > 0) {
+		if (rcvd_since_last_report > 0) {
 			thput = rcvd_since_last_report/time_for_rcv_rate;
 		}
 	}
@@ -326,6 +375,7 @@ void TfrcSinkAgent::nextpkt(double p) {
 	sendpkt(p);
 
 	/* schedule next report rtt/NumFeedback_ later */
+	/* note from Sally: why is this 1.5 instead of 1.0? */
 	if (rtt_ > 0.0 && NumFeedback_ > 0) 
 		nack_timer_.resched(1.5*rtt_/NumFeedback_);
 }
@@ -366,6 +416,11 @@ void TfrcSinkAgent::sendpkt(double p)
 			tfrc_ackh->flost = p;
 		tfrc_ackh->rate_since_last_report = est_thput ();
 		tfrc_ackh->losses = losses_since_last_report;
+		if (total_received_ <= 0) 
+			tfrc_ackh->true_loss = 0.0;
+		else 
+			tfrc_ackh->true_loss = 1.0 * 
+			    total_losses_/(total_received_+total_dropped_);
 		last_report_sent = now; 
 		rcvd_since_last_report = 0;
 		losses_since_last_report = 0;
@@ -392,6 +447,9 @@ int TfrcSinkAgent::command(int argc, const char*const* argv)
 			strcpy(w, (char *)argv[2]);
 			numsamples = atoi(strtok(w,"+"));
 			sample = (int *)malloc((numsamples+1)*sizeof(int));
+			losses = (int *)malloc((numsamples+1)*sizeof(int));
+                        count_losses = (int *)malloc((numsamples+1)*sizeof(int));
+                        num_rtts = (int *)malloc((numsamples+1)*sizeof(int));
 			weights = (double *)malloc((numsamples+1)*sizeof(double));
 			mult = (double *)malloc((numsamples+1)*sizeof(double));
 			fflush(stdout);
@@ -399,6 +457,9 @@ int TfrcSinkAgent::command(int argc, const char*const* argv)
 				int count = 0 ;
 				while (count < numsamples) {
 					sample[count] = 0;
+					losses[count] = 1;
+					count_losses[count] = 0;
+                                        num_rtts[count] = 0;
 					mult[count] = 1;
 					char *w;
 					w = strtok(NULL, "+");
@@ -413,6 +474,9 @@ int TfrcSinkAgent::command(int argc, const char*const* argv)
 					abort();
 				}
 				sample[count] = 0;
+				losses[count] = 1;
+				count_losses[count] = 0;
+				num_rtts[count] = 0;
 				weights[count] = 0;
 				mult[count] = 1;
 				free(w);
@@ -451,6 +515,27 @@ void TfrcSinkAgent::print_loss_all(int *sample)
 		now, sample[0], sample[1], sample[2], sample[3], sample[4]); 
 }
 
+void TfrcSinkAgent::print_losses_all(int *losses) 
+{
+	double now = Scheduler::instance().clock();
+	printf ("%f: losses 0: %5d 1: %5d 2: %5d 3: %5d 4: %5d\n", 
+		now, losses[0], losses[1], losses[2], losses[3], losses[4]); 
+}
+
+void TfrcSinkAgent::print_count_losses_all(int *count_losses) 
+{
+	double now = Scheduler::instance().clock();
+	printf ("%f: count? 0: %5d 1: %5d 2: %5d 3: %5d 4: %5d\n", 
+		now, count_losses[0], count_losses[1], count_losses[2], count_losses[3], count_losses[4]); 
+}
+
+void TfrcSinkAgent::print_num_rtts_all(int *count_losses) 
+{
+	double now = Scheduler::instance().clock();
+	printf ("%f: rtts 0: %5d 1: %5d 2: %5d 3: %5d 4: %5d\n", 
+ 	   now, num_rtts[0], num_rtts[1], num_rtts[2], num_rtts[3], num_rtts[4]); 
+}
+
 ////////////////////////////////////////
 // algo specific code /////////////////
 ///////////////////////////////////////
@@ -468,22 +553,41 @@ double TfrcSinkAgent::est_loss_WALI ()
 	if (!init_WALI_flag) {
 		init_WALI () ;
 	}
-	// sample[i] counts the number of packets since the i-th loss event
+	// sample[i] counts the number of packets in the i-th loss interval
 	// sample[0] contains the most recent sample.
+        // losses[i] contains the number of losses in the i-th loss interval
+        // count_losses[i] is 1 if the i-th loss interval is short.
+        // num_rtts[i] contains the number of rtts in the i-th loss interval
 	for (i = last_sample; i <= maxseq ; i ++) {
 		sample[0]++;
 		if (lossvec_[i%hsz] == LOST || lossvec_[i%hsz] == ECNLOST) {
 		        //  new loss event
-			// double now = Scheduler::instance().clock();
 			sample_count ++;
 			shift_array (sample, numsamples+1, 0); 
+			shift_array (losses, numsamples+1, 1); 
+			shift_array (count_losses, numsamples+1, 1); 
+			shift_array (num_rtts, numsamples+1, 0); 
 			multiply_array(mult, numsamples+1, mult_factor_);
 			shift_array (mult, numsamples+1, 1.0); 
 			mult_factor_ = 1.0;
 		}
 	}
 	last_sample = maxseq+1 ; 
-
+	double now = Scheduler::instance().clock();
+        //if (ShortIntervals_ > 0 && printLoss_ > 0) {
+        //    printf ("now: %5.2f lastloss: %5.2f ShortRtts_: %d rtt_: %5.2f\n",
+        //         now, lastloss, ShortRtts_, rtt_);
+        //}
+        if (ShortIntervals_ > 0 && 
+            now - lastloss > ShortRtts_ * rtt_) {
+              // Check if the current loss interval is short.
+              count_losses[0] = 0;
+        }
+        if (ShortIntervals_ > 0 && rtt_ > 0) {
+              // Count number of rtts in current loss interval.
+              num_rtts[0] = (int) ceil((now - lastloss) / rtt_);
+              if (num_rtts[0] < 1) num_rtts[0] = 1;
+        }
 	if (sample_count>numsamples+1)
 		// The array of loss intervals is full.
 		ds=numsamples+1;
@@ -495,7 +599,8 @@ double TfrcSinkAgent::est_loss_WALI ()
 		return 0; 
 	/* do we need to discount weights? */
 	if (sample_count > 1 && discount && sample[0] > 0) {
-		double ave = weighted_average(1, ds, 1.0, mult, weights, sample);
+                //double ave = weighted_average1(1, ds, 1.0, mult, weights, sample, ShortIntervals_, losses, count_losses);
+                double ave = weighted_average1(1, ds, 1.0, mult, weights, sample, ShortIntervals_, losses, count_losses, num_rtts);
 		int factor = 2;
 		double ratio = (factor*ave)/sample[0];
 		double min_ratio = 0.5;
@@ -507,24 +612,38 @@ double TfrcSinkAgent::est_loss_WALI ()
 		}
 	}
 	// Calculations including the most recent loss interval.
-	ave_interval1 = weighted_average(0, ds, mult_factor_, mult, weights, sample);
+        ave_interval1 = weighted_average1(0, ds, mult_factor_, mult, weights, sample, ShortIntervals_, losses, count_losses, num_rtts);
+        // Calculations not including the most recent loss interval.
+        ave_interval2 = weighted_average1(1, ds, mult_factor_, mult, weights, sample, ShortIntervals_, losses, count_losses, num_rtts);
 	// The most recent loss interval does not end in a loss
 	// event.  Include the most recent interval in the 
 	// calculations only if this increases the estimated loss
-	// interval.
-	ave_interval2 = weighted_average(1, ds, mult_factor_, mult, weights, sample);
-	if (ave_interval2 > ave_interval1)
+	// interval. 
+        // If ShortIntervals is less than 10, do not count the most
+        //   recent interval if it is a short interval.
+        //   Values of ShortIntervals greater than 10 are only for
+        //   validation purposes, and for backwards compatibility.
+        //
+	if (ave_interval2 > ave_interval1 ||
+             (ShortIntervals_ > 1 && ShortIntervals_ < 10 
+                     && count_losses[0] == 1))
+                // The second condition is to check if the first interval
+                //  is a short interval.  If so, we must use ave_interval2.
 		ave_interval1 = ave_interval2;
 	if (ave_interval1 > 0) { 
 		if (printLoss_ > 0) {
 			print_loss(sample[0], ave_interval1);
 			print_loss_all(sample);
+			if (ShortIntervals_ > 0) {
+				print_losses_all(losses);
+				print_count_losses_all(count_losses);
+                                print_num_rtts_all(num_rtts);
+			}
 		}
 		return 1/ave_interval1; 
 	} else return 999;     
 }
 
-
 // Calculate the weighted average.
 double TfrcSinkAgent::weighted_average(int start, int end, double factor, double *m, double *w, int *sample)
 {
@@ -565,6 +684,127 @@ double TfrcSinkAgent::weighted_average(int start, int end, double factor, double
 	}
 }
 
+int TfrcSinkAgent::get_sample(int oldSample, int numLosses) 
+{
+	int newSample;
+	if (numLosses == 0) {
+		newSample = oldSample;
+	} else {
+		newSample = oldSample / numLosses;
+	}
+	return newSample;
+}
+
+int TfrcSinkAgent::get_sample_rtts(int oldSample, int numLosses, int rtts) 
+{
+	int newSample;
+	if (numLosses == 0) {
+		newSample = oldSample;
+                //printf ("sample: %d numLosses: %d\n", oldSample, numLosses);
+	} else {
+                double fraction;
+                if (ShortRtts_ != 0)
+                     fraction = (ShortRtts_ + 1.0 - rtts) / ShortRtts_;
+                else fraction = 1.0;
+		int numLoss = (int) (floor(fraction * numLosses ));
+                if (numLoss != 0)
+		  newSample = oldSample / numLoss;
+                else newSample = oldSample;
+                //printf ("sample: %d rtts: %d numLosses: %d newSample: %d fraction: %5.2f numLoss %d\n",
+                //  oldSample, rtts, numLosses, newSample, fraction, numLoss);
+	}
+	return newSample;
+}
+
+// Calculate the weighted average, factor*m[i]*w[i]*sample[i]/wsum.
+// "factor" is "mult_factor_", for weighting the most recent interval
+//    when it is very large
+// "m[i]" is "mult[]", for old values of "mult_factor_".
+//
+// When ShortIntervals_%10 is 1, the length of a loss interval is
+//   "sample[i]/losses[i]" for short intervals, not just "sample[i]".
+//   This is equivalent to a loss event rate of "losses[i]/sample[i]",
+//   instead of "1/sample[i]".
+//
+// When ShortIntervals_%10 is 2, it is like ShortIntervals_ of 1,
+//   except that the number of losses per loss interval is at
+//   most 1460/byte-size-of-small-packets.
+//
+// When ShortIntervals_%10 is 3, short intervals are up to three RTTs,
+//   and the number of losses counted is a function of the interval size.
+//
+double TfrcSinkAgent::weighted_average1(int start, int end, double factor, double *m, double *w, int *sample, int ShortIntervals, int *losses, int *count_losses, int *num_rtts)
+{
+        int i;
+        int ThisSample;
+        double wsum = 0;
+        double answer = 0;
+        if (smooth_ == 1 && start == 0) {
+                if (end == numsamples+1) {
+                        // the array is full, but we don't want to use
+                        //  the last loss interval in the array
+                        end = end-1;
+                }
+                // effectively shift the weight arrays
+                for (i = start ; i < end; i++)
+                        if (i==0)
+                                wsum += m[i]*w[i+1];
+                        else
+                                wsum += factor*m[i]*w[i+1];
+                for (i = start ; i < end; i++) {
+                        ThisSample = sample[i];
+                        if (ShortIntervals%10 == 1 && count_losses[i] == 1) {
+			       ThisSample = get_sample(sample[i], losses[i]);
+                        }
+                        if (ShortIntervals%10 == 2 && count_losses[i] == 1) {
+			       int adjusted_losses = int(fsize_/size_);
+			       if (losses[i] < adjusted_losses) {
+					adjusted_losses = losses[i];
+			       }
+			       ThisSample = get_sample(sample[i], adjusted_losses);
+                        }
+                        if (ShortIntervals%10 == 3 && count_losses[i] == 1) {
+			       ThisSample = get_sample_rtts(sample[i], losses[i], num_rtts[i]);
+                        }
+                        if (i==0)
+                                answer += m[i]*w[i+1]*ThisSample/wsum;
+                                //answer += m[i]*w[i+1]*sample[i]/wsum;
+                        else
+                                answer += factor*m[i]*w[i+1]*ThisSample/wsum;
+                                //answer += factor*m[i]*w[i+1]*sample[i]/wsum;
+		}
+                return answer;
+
+        } else {
+                for (i = start ; i < end; i++)
+                        if (i==0)
+                                wsum += m[i]*w[i];
+                        else
+                                wsum += factor*m[i]*w[i];
+                for (i = start ; i < end; i++) {
+                       ThisSample = sample[i];
+                       if (ShortIntervals%10 == 1 && count_losses[i] == 1) {
+			       ThisSample = get_sample(sample[i], losses[i]);
+                       }
+                       if (ShortIntervals%10 == 2 && count_losses[i] == 1) {
+			       ThisSample = get_sample(sample[i], 7);
+			       // Replace 7 by 1460/packet size.
+                               // NOT FINISHED.
+                       }
+                        if (ShortIntervals%10 == 3 && count_losses[i] == 1) {
+			       ThisSample = get_sample_rtts(sample[i], losses[i], (int) num_rtts[i]);
+                        }
+                       if (i==0)
+                                answer += m[i]*w[i]*ThisSample/wsum;
+                                //answer += m[i]*w[i]*sample[i]/wsum;
+                        else
+                                answer += factor*m[i]*w[i]*ThisSample/wsum;
+                                //answer += factor*m[i]*w[i]*sample[i]/wsum;
+		}
+                return answer;
+        }
+}
+
 // Shift array a[] up, starting with a[sz-2] -> a[sz-1].
 void TfrcSinkAgent::shift_array(int *a, int sz, int defval) 
 {
@@ -606,13 +846,24 @@ double TfrcSinkAgent::adjust_history (double ts)
 	}
 	lastloss = ts; 
 	lastloss_round_id = round_id ;
-	p=b_to_p(est_thput()*psize_, rtt_, tzero_, psize_, 1);
+	p=b_to_p(est_thput()*psize_, rtt_, tzero_, fsize_, 1);
 	false_sample = (int)(1.0/p);
 	sample[1] = false_sample;
 	sample[0] = 0;
+	losses[1] = 0;
+	losses[0] = 1;
+	count_losses[1] = 0;
+	count_losses[0] = 0;
+        num_rtts[0]=0;
+        num_rtts[1]=0;
 	sample_count++; 
 	if (printLoss_) {
 		print_loss_all (sample);
+		if (ShortIntervals_ > 0) {
+			print_losses_all(losses);
+			print_count_losses_all(count_losses);
+			print_num_rtts_all(num_rtts);
+		}
 	}
 	false_sample = -1 ; 
 	return p;
@@ -630,6 +881,9 @@ void TfrcSinkAgent::init_WALI () {
 		numsamples = numsamples + 1;
 	}
 	sample = (int *)malloc((numsamples+1)*sizeof(int));
+        losses = (int *)malloc((numsamples+1)*sizeof(int));
+        count_losses = (int *)malloc((numsamples+1)*sizeof(int));
+        num_rtts = (int *)malloc((numsamples+1)*sizeof(int));
 	weights = (double *)malloc((numsamples+1)*sizeof(double));
 	mult = (double *)malloc((numsamples+1)*sizeof(double));
 	for (i = 0 ; i < numsamples+1 ; i ++) {
@@ -682,7 +936,16 @@ double TfrcSinkAgent::est_loss_EWMA () {
 	} else {
 		p1 = 1.0/avg_loss_int ; 
 	}
-	if (loss_int == 0) {
+	if (loss_int == 0 
+	    || avg_loss_int < 0){ //XXX this last check was added by a
+				  //person who knows nothing of this
+				  //code just to stop FP div by zero.
+				  //Values were history=.75,
+				  //avg_loss_int=-1, loss_int=3.  If
+				  //you know what should be here,
+				  //please cleanup and remove this
+				  //comment.
+
 		p2 = p1 ; 
 	} else {
 		p2 = 1.0/(history*avg_loss_int + (1-history)*loss_int) ;
