@@ -1,21 +1,44 @@
-// Copyright (c) Xerox Corporation 1998. All rights reserved.
-//
-// License is granted to copy, to use, and to make and to use derivative
-// works for research and evaluation purposes, provided that Xerox is
-// acknowledged in all documentation pertaining to any such copy or
-// derivative work. Xerox grants no other licenses expressed or
-// implied. The Xerox trade name should not be used in any advertising
-// without its written permission. 
-//
-// XEROX CORPORATION MAKES NO REPRESENTATIONS CONCERNING EITHER THE
-// MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF THIS SOFTWARE
-// FOR ANY PARTICULAR PURPOSE.  The software is provided "as is" without
-// express or implied warranty of any kind.
-//
-// These notices must be retained in any copies of any part of this
-// software. 
-//
-// $Header: /nfs/jade/vint/CVSROOT/ns-2/webcache/pagepool.cc,v 1.14 1999/08/04 21:04:04 haoboy Exp $
+/*
+ * Copyright (c) Xerox Corporation 1998. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2 of the License, or (at your
+ * option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * Linking this file statically or dynamically with other modules is making
+ * a combined work based on this file.  Thus, the terms and conditions of
+ * the GNU General Public License cover the whole combination.
+ *
+ * In addition, as a special exception, the copyright holders of this file
+ * give you permission to combine this file with free software programs or
+ * libraries that are released under the GNU LGPL and with code included in
+ * the standard release of ns-2 under the Apache 2.0 license or under
+ * otherwise-compatible licenses with advertising requirements (or modified
+ * versions of such code, with unchanged license).  You may copy and
+ * distribute such a system following the terms of the GNU GPL for this
+ * file and the licenses of the other code concerned, provided that you
+ * include the source code of that other code when and as the GNU GPL
+ * requires distribution of source code.
+ *
+ * Note that people who make modified versions of this file are not
+ * obligated to grant this special exception for their modified versions;
+ * it is their choice whether to do so.  The GNU General Public License
+ * gives permission to release a modified version without this exception;
+ * this exception also makes it possible to release a modified version
+ * which carries forward this exception.
+ *
+ * $Header: /cvsroot/nsnam/ns-2/webcache/pagepool.cc,v 1.17 2006/02/21 15:20:20 mahrenho Exp $
+ */
 
 #include <stdlib.h>
 #include <sys/types.h>
@@ -257,8 +280,9 @@ int TracePagePool::add_page(const char* name, ServerPage *pg)
 		fprintf(stderr, "TracePagePool: Duplicate entry %s\n", 
 			name);
 
+	long key = pg->id();
 	Tcl_HashEntry *hf = 
-		Tcl_CreateHashEntry(idmap_, (const char *)pg->id(), &newEntry);
+		Tcl_CreateHashEntry(idmap_, (const char *)key, &newEntry);
 	if (hf == NULL) {
 		Tcl_DeleteHashEntry(he);
 		return -1;
@@ -276,7 +300,8 @@ ServerPage* TracePagePool::get_page(int id)
 {
 	if ((id < 0) || (id >= num_pages_))
 		return NULL;
-	Tcl_HashEntry *he = Tcl_FindHashEntry(idmap_, (const char *)id);
+	long key = id;
+	Tcl_HashEntry *he = Tcl_FindHashEntry(idmap_, (const char *)key);
 	if (he == NULL)
 		return NULL;
 	return (ServerPage *)Tcl_GetHashValue(he);
@@ -653,12 +678,9 @@ int ClientPagePool::command(int argc, const char*const* argv)
 ClientPage* ClientPagePool::get_page(const char *name)
 {
 	PageID t1;
-	void* t2[2];
 	ClientPage::split_name(name, t1);
-	t2[0] = (void *)t1.s_;
-	t2[1] = (void *)t1.id_;
 
-	Tcl_HashEntry *he = Tcl_FindHashEntry(namemap_, (const char *)t2);
+	Tcl_HashEntry *he = Tcl_FindHashEntry(namemap_, (const char *)&t1);
 	if (he == NULL)
 		return NULL;
 	return (ClientPage *)Tcl_GetHashValue(he);
@@ -744,14 +766,11 @@ int ClientPagePool::add_page(ClientPage* pg)
 	pg->name(buf);
 
 	PageID t1;
-	void* t2[2];
 	ClientPage::split_name(buf, t1);
-	t2[0] = (void *)t1.s_;
-	t2[1] = (void *)t1.id_;
 
 	int newEntry = 1;
 	Tcl_HashEntry *he = Tcl_CreateHashEntry(namemap_, 
-						(const char *)t2,
+						(const char *)&t1,
 						&newEntry);
 	if (he == NULL)
 		return -1;
@@ -778,13 +797,10 @@ int ClientPagePool::add_page(ClientPage* pg)
 int ClientPagePool::remove_page(const char *name)
 {
 	PageID t1;
-	void* t2[2];
 	ClientPage::split_name(name, t1);
-	t2[0] = (void *)t1.s_;
-	t2[1] = (void *)t1.id_;
 
 	// Find out which client we are seeking
-	Tcl_HashEntry *he = Tcl_FindHashEntry(namemap_, (const char *)t2);
+	Tcl_HashEntry *he = Tcl_FindHashEntry(namemap_, (const char *)&t1);
 	if (he == NULL)
 		return -1;
 	ClientPage *pg = (ClientPage *)Tcl_GetHashValue(he);
@@ -945,7 +961,7 @@ int ProxyTracePagePool::find_info()
 		abort();
 	}
 	double len;
-	sscanf(buf+i+1, "%lf %u", &len, &num_pages_);
+	sscanf(buf+i+1, "%lf %i", &len, &num_pages_);
 	duration_ = (int)ceil(len);
 #if 0
 	printf("ProxyTracePagePool: duration %d pages %u\n",
@@ -980,12 +996,12 @@ ProxyTracePagePool::ClientRequest* ProxyTracePagePool::load_req(int cid)
 	Tcl_HashEntry *he;
 	ClientRequest *p;
 	int dummy; 
-	
-	if ((he = Tcl_FindHashEntry(req_, (const char*)cid)) == NULL) {
+	long key = cid;
+	if ((he = Tcl_FindHashEntry(req_, (const char*)key)) == NULL) {
 		// New entry
 		p = new ClientRequest();
 		p->seq_ = lastseq_++;
-		he = Tcl_CreateHashEntry(req_, (const char*)cid, &dummy);
+		he = Tcl_CreateHashEntry(req_, (const char*)key, &dummy);
 		Tcl_SetHashValue(he, (const char*)p);
 		// Search from the beginning of file for this new client
 		fseek(reqfile_, 0, SEEK_SET);
